import wxversion
wxversion.select('2.8')

import wx

from OpenGL.GLUT import *

from display.window.colorkeyview import *

from display.window.alnview import *
from display.window.featureview import *

from feature.cddfetcher import *
from feature.bindnfetcher import *
from feature.drawablefeature import *

from display.applicationhelper import *

from color import *

from util.preferences import *

class Application(wx.App):
    def __init__(self, width, height, title):
        self.__width = width
        self.__height = height
        self.__title = title
        
        wx.App.__init__(self, 0)
        
        glutInit([])
        
        setApplication(self)
    
    def OnInit(self):
        self.__frame = Frame(None, self.__width, self.__height, self.__title)
        self.SetTopWindow(self.__frame)
        self.__frame.Show(True)
        
        self.__frame.setInitialSize()
        
        return True
    
    def getMainFrame(self):
        return self.__frame

class MenuBar(wx.MenuBar):
    # File Menu
    LOADFEATUREALN = 1002
    CLEARALN = 1004
    EXIT = 1001
    LOADVIEWINGALN = 1006
    
    # Edit Menu
    PREFERENCES = 1005
    
    # Help Menu
    ABOUT = 1003
    HELPTOPICS = 1007
    
    def __init__(self, style = 0):
        wx.MenuBar.__init__(self, style)
        
        fileMenu = wx.Menu()
        fileMenu.Append(MenuBar.LOADFEATUREALN, 'Load Alignment in &Domains View...\tCtrl+O', 'Load Alignment from File')
        #fileMenu.Append(MenuBar.LOADVIEWINGALN, 'Load Alignment in &Traditional View...\tCtrl+T', 'Load Alignment from File')
        #fileMenu.Append(MenuBar.CLEARALN, '&Clear Alignment', 'Clear Alignment from View')
        fileMenu.AppendSeparator()
        fileMenu.Append(MenuBar.EXIT, 'E&xit\tCtrl+Q', 'Exit Application')
        self.Append(fileMenu, '&File')
        
        #editMenu = wx.Menu()
        #editMenu.Append(MenuBar.PREFERENCES, '&Preferences', 'Preferences')
        #self.Append(editMenu, '&Edit')
        
        helpMenu = wx.Menu()
        helpMenu.Append(MenuBar.ABOUT, '&About...', 'About Application')
        helpMenu.Append(MenuBar.HELPTOPICS, '&Help Topics...\tF1', 'Help')
        self.Append(helpMenu, '&Help')
        
class Frame(wx.Frame):
    SETSIZETOWIN = 1
    SETSIZETOSIZER = 2
    
    def __init__(self, parent, width, height, title):
        wx.Frame.__init__(self, parent, -1, title, size = (width, height), style = wx.DEFAULT_FRAME_STYLE &~ wx.MAXIMIZE_BOX)
        
        # Menubar
        self.__menuBar = MenuBar()
        self.SetMenuBar(self.__menuBar)
        self.Bind(wx.EVT_MENU, self.OnMenuExit, id = MenuBar.EXIT)
        self.Bind(wx.EVT_MENU, self.OnLoadFeatureAln, id = MenuBar.LOADFEATUREALN)
        self.Bind(wx.EVT_MENU, self.OnAbout, id = MenuBar.ABOUT)
        self.Bind(wx.EVT_MENU, self.OnClearAln, id = MenuBar.CLEARALN)
        self.Bind(wx.EVT_MENU, self.OnPreferences, id = MenuBar.PREFERENCES)
        self.Bind(wx.EVT_MENU, self.OnLoadViewingAln, id = MenuBar.LOADVIEWINGALN)
        
        self.__aboutDialogInfo = wx.AboutDialogInfo()
        self.__aboutDialogInfo.AddDeveloper("Andrew D. Lindeman <ADL91@msstate.edu>")
        self.__aboutDialogInfo.AddDeveloper("Susan M. Bridges <bridges@cse.msstate.edu>")
        self.__aboutDialogInfo.AddDeveloper("T.J. Jankun-Kelly <tjk@cse.msstate.edu>")
        self.__aboutDialogInfo.SetName("MSAVIS")
        self.__aboutDialogInfo.SetVersion("0.9 alpha")
        self.__aboutDialogInfo.SetWebSite("http://research.adml.us/msavis")
        
        self.__windows = []
        self.__alnWindows = {}
        
        self.__alnZooms = {}
        self.__alnSeqOrders = {}
        
        self.__noOnSizeLayout = False
        
        self.__sizer = wx.BoxSizer(wx.VERTICAL)
        #self.__sizer = wx.FlexGridSizer(cols = 1, vgap = 10)
        #self.__sizer.SetFlexibleDirection(wx.BOTH)
        #self.__sizer.AddGrowableCol(0)
        self.SetSizer(self.__sizer)
                
        # Color
        grey = Color(Color.RGB, 0.85, 0.85, 0.85)
        grey.provideToFunctionAsWxColour(self.SetBackgroundColour)
        self.ClearBackground()                
         
        #self.SetAutoLayout(True)
            
        self.setMaximizedState()
                
        self.Bind(wx.EVT_SIZE, self.OnSize)
        self.Bind(wx.EVT_MAXIMIZE, self.OnMaximize)
        
    def setMaximizedState(self):
        self.__maximized = self.IsMaximized()
        
    def OnSize(self, event):
        size = self.GetSize()
        
        self.doLayout(self.__maximized) # Don't fit if not coming back from being maximized
        self.setMaximizedState()
            
        event.Skip()
        
    def OnClearAln(self, event):
        setAlignment(None)
        
    def OnAbout(self, event):
        wx.AboutBox(self.__aboutDialogInfo)
        
    def OnPreferences(self, event):
        aln = getAlignment()
        frameView = self.getFrameView()
        
        #prefDialog = FetchFeaturesDialog(self, -1, disableInitialPreferences = True)
        #prefDialog.ShowModal()
        
        # Things that can change:
        
        # Scoring method
        if aln is not None:
            aln.clearColumnScores()
        
        # Need to redraw?
        frameView.handlePreferencesUpdate()
        
    def OnLoadViewingAln(self, event):
        appHelper = getApplicationHelper()
        wxHelper = getWxHelper()
        
        if not self.askToRemoveCurrentWindows():
            return
        
        files = wxHelper.getAlnFilesFromDialog(self, "Open Alignment File (Traditional View)", None, None, None, wx.FD_OPEN)
        if len(files) != 1:
            return
        
        filename = files[0]
        aln = self.attemptLoadAlignment(filename)
        if aln is None:
            return
        
        self.loadAlignmentView(aln)
        
    def attemptLoadAlignment(self, filename):
        wxHelper = getWxHelper()
        appHelper = getApplicationHelper()
        
        type = AlignmentManager.INVALID
        ext = appHelper.getFileExtension(filename).lower()
        if ext == 'aln':
            type = AlignmentManager.CLUSTALW
        elif ext == 'phylip':
            type = AlignmentManager.PHYLIP
        
        aln = AlignmentManager()
        try:
            aln.loadAlignmentFromFile(type, filename)
        except:
            del aln
            wxHelper.displayErrorMessage(self, "Cannot load alignment!  Check its format!", "Error Loading Alignment")
            return None
        
        return aln
        
    def OnLoadFeatureAln(self, event):
        appHelper = getApplicationHelper()
        helper = getWxHelper()
        preferences = getPreferences()
        
        if not self.askToRemoveCurrentWindows():
            return
        
        files = helper.getAlnFilesFromDialog(self, "Open Alignment File (Feature View)", None, None, None, wx.FD_OPEN)
        if len(files) != 1:
            return
        
        filename = files[0]    
        aln = self.attemptLoadAlignment(filename)
        if aln is None:
            return

        self.__progressDlg = wx.ProgressDialog("Downloading Features", "Downloading Features from Internet", maximum = (aln.getNumSequences() - 1), style = wx.PD_APP_MODAL | wx.PD_AUTO_HIDE | wx.PD_ELAPSED_TIME)            
        self.setFetcherProgress(0)
    
        aln.fetchFeatures(preferences.getEnabledFetchers())

        self.__progressDlg.Destroy()
        self.__progressDlg = None
    
        if len(aln.getUniqueFeatureData()) == 0:
            helper.displayErrorMessage(self, "No features found for alignment!\n\nResults view will not be displayed")
        else:
            self.loadFeatureView(aln)
    
    def askToRemoveCurrentWindows(self):
        if self.hasWindows():
            wxHelper = getWxHelper()
            resp = wxHelper.displayYesNoMessage(self, "Remove current view?")
            
            if resp == wx.ID_YES:
                self.removeAllWindows()
                return True
            else:
                return False
        else:
            return True
    
    def hasWindows(self):
        if len(self.__windows) > 0:
            return True
        else:
            return False
    
    def loadFeatureView(self, aln):
        window = FeatureView(self, -1, style = wx.TAB_TRAVERSAL, aln = aln, zoom = self.getZoomForAlignment(aln), seqOrder = self.getSequenceOrderForAlignment(aln))
        self.addWindow(window, aln)
    
    def loadAlignmentView(self, aln):
        window = AlnView(self, -1, style = wx.TAB_TRAVERSAL, aln = aln, zoom = self.getZoomForAlignment(aln), seqOrder = self.getSequenceOrderForAlignment(aln))
        self.addWindow(window, aln)
        
    def addWindow(self, win, aln = None, addToSizer = True):
        self.__windows.append(win)
            
        if addToSizer:
            self.__sizer.Add(win, 1, wx.EXPAND)
            
            #self.doLayout(self.getTotalManagedSize())
            self.doLayout()
            self.__sizer.Fit(self)
        
        if aln is not None:
            if aln not in self.__alnWindows:
                self.__alnWindows[aln] = []
            
            self.__alnWindows[aln].append(win)
            
        self.CenterOnScreen()

    def removeWindow(self, win, destroy = True, removeAllAlnWindows = True):
        if win in self.__windows:
            self.__windows.remove(win)
        
        inAln = []
        for aln in self.__alnWindows:
            if win in self.__alnWindows[aln]:
                self.__alnWindows[aln].remove(win)
                
                if removeAllAlnWindows:
                    for alnWin in self.__alnWindows[aln]:
                        self.removeWindow(alnWin, destroy, removeAllAlnWindows)
        
        self.__sizer.Remove(win)
        self.doLayout()
        
        if destroy:
            win.DestroyChildren()
            win.Destroy()
            del win
        
    def removeAllWindows(self, destroy = True):
        for win in self.__windows:
            self.removeWindow(win, destroy)
       
    def getWindowsForAlignment(self, aln):
        if aln in self.__alnWindows:
            return self.__alnWindows[aln]
        else:
            return []
        
    def getZoomForAlignment(self, aln):
        if aln not in self.__alnZooms:
            self.setZoomForAlignment(aln, 0, aln.getLength(), False)
        
        return self.__alnZooms[aln]

    def setZoomForAlignment(self, aln, start, end, updateWindows = True):
        self.__alnZooms[aln] = (start, end)        
        
        if updateWindows:
            self.setZoomForAlignmentWindows(aln)
        
    def setZoomForAlignmentWindows(self, aln):
        zoom = self.getZoomForAlignment(aln)
        windows = self.getWindowsForAlignment(aln)
        
        for win in windows:
            if isinstance(win, Zoomable):
                win.setZoom(zoom[0], zoom[1], True)
            
    def getSequenceOrderForAlignment(self, aln):
        if aln not in self.__alnSeqOrders:
            self.__alnSeqOrders[aln] = range(aln.getNumSequences())
            
        return self.__alnSeqOrders[aln]
    
    def swapSequenceOrderForAlignmentWindows(self, aln, one, two):
        windows = self.getWindowsForAlignment(aln)
        
        for win in windows:
            if isinstance(win, SequenceSwapable):
                win.swapSequenceOrder(one, two, True)
    
    def setNotifySequence(self, aln, index):
        for win in self.getWindowsForAlignment(aln):
            if isinstance(win, SequenceNotifiable):
                win.setNotifySequence(index, True)
    
    def swapSequenceOrderForAlignment(self, aln, one, two, updateWindows = True):
        indexOne = self.__alnSeqOrders[aln].index(one)
        indexTwo = self.__alnSeqOrders[aln].index(two)
        
        self.__alnSeqOrders[aln][indexOne] = two
        self.__alnSeqOrders[aln][indexTwo] = one
        
        if updateWindows:
            self.swapSequenceOrderForAlignmentWindows(aln, one, two)
            
    def setFetcherProgress(self, num):
        self.__progressDlg.Update(num - 1)
   
    def OnMenuExit(self, event):
        self.Close(True)
        exit()
        
    def adjustSize(self, deltaX, deltaY):
        # TODO: Check that this actually works
        size = self.GetSize()
        
        self.SetSize(size.width + deltaX, size.height + deltaY)
        
        self.doLayout()
        
    def OnMaximize(self, event):
        self.doLayout()
        
        self.setMaximizedState()
        
        event.Skip()
        
    def doLayout(self, fit = True):
        self.__sizer.Layout()
        
        if not self.IsMaximized() and fit:
            if self.hasWindows():
                self.__sizer.Fit(self)
            else:
                self.setInitialSize()
    
    def setInitialSize(self):
        preferences = getPreferences()
        self.SetSize(preferences.getFrameInitialSize())
        self.doLayout(False)
        self.CenterOnScreen()        
        
    def getTotalManagedSize(self):
        children = self.__sizer.GetChildren()
        
        width = 0
        height = 0
        for childSizerItem in children:
            win = childSizerItem.GetWindow()
            
            if win is not None:
                size = win.GetClientSize()
                width += size.width
                height += size.height
                
        return wx.Size(width, height)