from display.color import *

from display.applicationhelper import *

class ColorKey:
    __colors = [
                [[234, 189, 184], [206, 162, 159], [196, 129, 126], [169, 103, 100]], # 0
                [[214, 200, 144], [188, 173, 120], [168, 146, 68], [142, 119, 43]], # 1
                [[208, 194, 220], [182, 167, 193], [162, 137, 178], [137, 111, 152]], # 2
                [[158, 211, 183], [133, 184, 158], [84, 162, 126], [58,135, 103]], # 3
                [[156, 208, 220], [131, 181, 193], [74, 158, 179], [45, 131, 153]], # 4
                [[233, 191, 162], [205, 165, 137], [191, 134, 94], [164, 108, 69]], # 5
                [[193, 205, 149], [166, 179, 126], [135, 155, 79], [109, 129, 55]], # 6
                [[150, 211, 201], [125, 184, 176], [63, 162, 153], [35, 135, 128]], # 7
                [[226, 190, 201], [199, 164, 175], [188, 130, 151], [161, 104, 126]], # 8
                [[183, 201, 226], [158, 174, 199], [122, 148, 187], [96, 122, 162]] # 9
            ]
    
    __backgroundBase = [
                        [243, 243, 243],
                        [215, 215, 215],
                        [203, 203, 203],
                        [176, 176, 176]
                    ] 
    
    __convertFromArray = True
    
    def colorArrayToColorObj(self, array):
        return Color(Color.RGB255, array[0], array[1], array[2])
    
    def __init__(self, aln, interpolateTo = 256):
        self.__key = {}
        
        self.__colorAt = 0
        
        # Interpolate if needed
        appHelper = getApplicationHelper()
        
        if self.__convertFromArray:
            newColors = []
            for i in range(len(self.__colors)):
                newColors.append([])
                
                for colorArray in self.__colors[i]:
                    newColors[i].append(self.colorArrayToColorObj(colorArray))
            
            self.__colors = newColors
            
            newBGColors = []
            for colorArray in self.__backgroundBase:
                newBGColors.append(self.colorArrayToColorObj(colorArray))
                
            self.__backgroundBase = newBGColors
        
        if interpolateTo != -1:
            for i in range(len(self.__colors)):
                colors = self.__colors[i]
                if len(colors) != interpolateTo and interpolateTo > len(colors):
                    self.__colors[i] = appHelper.interpolateColorsToResolution(colors, interpolateTo)
                    
            if len(self.__backgroundBase) != interpolateTo and interpolateTo > len(self.__backgroundBase):
                self.__backgroundBase = appHelper.interpolateColorsToResolution(self.__backgroundBase, interpolateTo)
        
    def get(self, key):
        if key not in self.__key:
            self.__key[key] = self.__colors[self.__colorAt % len(self.__colors)]
            self.__colorAt += 1

        return self.__key[key]
    
    def getByOffset(self, colorOffset):
        return self.__colors[colorOffset]
    
    def getBackgroundBase(self):
        return self.__backgroundBase
    
    def getNumColors(self):
        return len(self.__colors)
    
colorKey = {}
def getColorKey(aln):
    global colorKey
    if aln not in colorKey:
        colorKey[aln] = ColorKey(aln)
        
    return colorKey[aln]
