import wx

from display.applicationhelper import *
from display.wxhelper import *

from util.preferences import *

class AddTraditionalTrackDialog(wx.Dialog):
    def __init__(self, parent, id, title = "Add Feature Track", pos = wx.DefaultPosition, size = wx.DefaultSize, style = wx.DEFAULT_DIALOG_STYLE, aln = None, alnView = None):
        wx.Dialog.__init__(self, parent, id, title, pos, size, style)
        
        self.__aln = aln
        self.__alnView = alnView
        
        appHelper = getApplicationHelper()
        preferences = getPreferences()
        wxHelper = getWxHelper()
        
        self.__sizer = wx.BoxSizer(wx.VERTICAL)
        
        ## Title
        self.__sizer.Add(wxHelper.makeBoldStaticText(self, -1, "Add Feature Track"), 0)
        
        ## Spacer
        self.__sizer.Add((0, 10), 0)
        
        ## Which sequence?
        self.__sizer.Add(wxHelper.makeBoldStaticText(self, -1, "Which Sequence?"), 0)
        
        self.__sequenceChoiceBox = wx.Choice(self, -1)
        for i in range(self.__aln.getNumSequences()):
            label = self.__aln.getSeqDescriptionByIndex(i)
            
            self.__sequenceChoiceBox.Append(label)
            
        self.__sizer.Add(self.__sequenceChoiceBox)
        
        ## Spacer
        self.__sizer.Add((0, 10), 0)
        
        ## Internet fetchers
        self.__sizer.Add(wxHelper.makeBoldStaticText(self, -1, "Internet Fetchers"), 0)
        
        self.__internetFetcherSizer = wx.GridSizer(cols = 2)
        self.__fetcherRadios = {}
        first = True
        
        self.__fetcherSelected = None
        
        for fetcher in preferences.getAllFetchers():
            if not fetcher.fetchesTrackFeature():
                continue
            
            style = 0
            if first:
                style = wx.RB_GROUP
            
            label = fetcher.getName()
            radioButton = wx.RadioButton(self, -1, label, style = style)
            radioButton.Bind(wx.EVT_RADIOBUTTON, self.OnFetcherRadio)
  
            self.__fetcherRadios[label] = radioButton
            self.__internetFetcherSizer.Add(radioButton, 1, wx.EXPAND)
            
            if first:
                radioButton.SetValue(True)
                self.__fetcherSelected = fetcher
                
                first = False
        
        self.__sizer.Add(self.__internetFetcherSizer, 1, wx.EXPAND)
        
        ## Spacer
        self.__sizer.Add((0, 10), 0)
            
        ## Buttons
        
        self.__buttonSizer = self.CreateStdDialogButtonSizer(wx.OK | wx.CANCEL)
        self.__sizer.Add(self.__buttonSizer, 1, wx.EXPAND)
        
        self.__cancelled = True
        
        self.SetSizer(self.__sizer)
    
    def OnFetcherRadio(self, event):
        obj = event.GetEventObject()
        label = obj.GetLabel()
        
        preferences = getPreferences()
        self.__fetcherSelected = preferences.getFetcher(label)
    
    def getFetcherSelection(self):
        return self.__fetcherSelected
    
    def getSequenceSelection(self):
        return self.__sequenceChoiceBox.GetSelection()
    
    def OnNext(self, event):
        wxHelper = getWxHelper()
        
        sel = self.getSequenceSelection()
        if sel == -1:
            wxHelper.displayErrorMessage(self, "Please select a sequence!", "Error!")
            return
        
        self.__cancelled = False
        self.Show(False)
        self.Destroy()
    
    def OnCancel(self, event):
        self.__cancelled = True
        self.Show(False)
        self.Destroy()
        
    def wasCancelled(self):
        return self.__cancelled