import wx
import wx.lib.intctrl as intctrl

from display.applicationhelper import *
from display.wxhelper import *

class ZoomToRegionDialog(wx.Dialog):
    def __init__(self, parent, id, title = "Add Feature Track", pos = wx.DefaultPosition, size = wx.DefaultSize, style = wx.DEFAULT_DIALOG_STYLE, aln = None):
        wx.Dialog.__init__(self, parent, id, title, pos, size, style)
        
        self.__aln = aln
        
        self.__sizer = wx.FlexGridSizer(cols = 2, vgap = 2, hgap = 2)
        
        wxHelper = getWxHelper()
        
        frame = getApplicationFrame()
        currentZoom = frame.getZoomForAlignment(self.__aln)
        
        self.__sizer.Add(wxHelper.makeBoldStaticText(self, -1, "Start Position: "), 0)
        self.__startPositionCtrl = wx.SpinCtrl(self, -1)
        self.__startPositionCtrl.SetRange(1, self.__aln.getLength() - 1)
        self.__startPositionCtrl.SetValue(currentZoom[0] + 1)
        self.__startPositionCtrl.Bind(wx.EVT_SPIN, self.OnStartPositionCtrlChange)
        self.__sizer.Add(self.__startPositionCtrl, 0)
        
        self.__sizer.Add(wxHelper.makeBoldStaticText(self, -1, "End Position: "), 0)
        self.__endPositionCtrl = wx.SpinCtrl(self, -1)
        self.__endPositionCtrl.SetRange(2, self.__aln.getLength())
        self.__endPositionCtrl.SetValue(currentZoom[1])
        self.__endPositionCtrl.Bind(wx.EVT_SPIN, self.OnEndPositionCtrlChange)
        self.__sizer.Add(self.__endPositionCtrl, 0)

        self.__buttonSizer = wx.StdDialogButtonSizer()

        self.__goButton = wx.Button(self, wx.ID_OK)
        self.__goButton.SetDefault()        
        self.__goButton.Bind(wx.EVT_BUTTON, self.OnGoButton)
        self.__buttonSizer.SetAffirmativeButton(self.__goButton)
        
        self.__cancelButton = wx.Button(self, wx.ID_CANCEL)
        self.__cancelButton.Bind(wx.EVT_BUTTON, self.OnCancelButton)
        self.__buttonSizer.SetCancelButton(self.__cancelButton)
        
        self.__buttonSizer.Realize()
        
        self.__sizer.Add((0, 0), 0)
        self.__sizer.Add(self.__buttonSizer, 0)
        
        self.SetSizerAndFit(self.__sizer)
        
    def OnCancelButton(self, event):
        self.Show(False)
        self.Destroy()
        
    def OnGoButton(self, event):
        start = self.__startPositionCtrl.GetValue() - 1
        end = self.__endPositionCtrl.GetValue()
        
        wxHelper = getWxHelper()
        if end < start:
            wxHelper.displayErrorMessage(self, "Zoom range out of bounds.  Please try again")
            return
        
        frame = getApplicationFrame()
        frame.setZoomForAlignment(self.__aln, start, end, True)
        
        self.Show(False)
        self.Destroy()
        
    def OnStartPositionCtrlChange(self, event):
        self.__endPositionCtrl.SetRange(event.GetInt() + 1, self.__aln.getLength())
        
    def OnEndPositionCtrlChange(self, event):
        self.__startPositionCtrl.SetRange(1, event.GetInt() - 1)