import wx

from display.color import *
from display.wxhelper import *
from display.applicationhelper import *

from util.alignmentmanager import *

class AlnRuler(wx.Panel, Zoomable):
    def __init__(self, parent, id, pos = wx.DefaultPosition, size = wx.DefaultSize, zoom = None):
        self.__init = False
        wx.Panel.__init__(self, parent, id, pos, size)
        Zoomable.__init__(self, zoom)
        
        if zoom is None:
            self.setZoom(1, 1, False)
        
        self.SetMinSize((45, 21))
        self.__init = True
        
        self.Bind(wx.EVT_SIZE, self.OnSize)
        self.Bind(wx.EVT_PAINT, self.OnPaint)
    
    def determineNumTicks(self):
        size = self.GetClientSize()
        length = self.getZoomLength()
        
        # TODO: Arbitrary math
        ticks = size.width / 75
        
        if ticks > self.getZoomLength():
            ticks = self.getZoomLength()
        
        return max(ticks, 0)
    
    def OnPaint(self, event):
        dc = wx.PaintDC(self)
        
        size = self.GetClientSize()
        length = self.getZoomLength()
        blockWidth = float(size.width) / length
        
        textHeight = size.height - 8
        
        wxHelper = getWxHelper()
        color = Color(Color.RGB, 1, 0.7, 0.7)
        font = wx.FontFromPixelSize((.45 * textHeight, textHeight), wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL)

        ticks = self.determineNumTicks()

        # Draw line        
        fiveUp = size.height - 5
        
        pt1 = wx.Point(0, fiveUp)
        pt2 = wx.Point(size.width, fiveUp)
        wxHelper.drawLine(dc, pt1, pt2, color)        
        
        tickList = self.cutTicks(self.start, self.end, ticks)
        
        addedStartStop = False
        if self.start not in tickList and self.end not in tickList:
            addedStartStop = True
            tickList.append(self.start)
            tickList.append(self.end)
        
        for tick in tickList:
            tickAdj = tick - self.start
            
            pt = pt1 = pt2 = label = None
            
            if addedStartStop and tick == self.start:
                label = str(tick + 1)
                
                pt = wx.Point(0, 0)
                
                pt1 = wx.Point(0, fiveUp - 2)
                pt2 = wx.Point(0, fiveUp + 2)
            elif addedStartStop and tick == self.end:
                label = str(tick)
                
                width, height = wxHelper.getTextExtent(dc, font, label)
                pt = wx.Point(size.width - width, 0)
                
                pt1 = wx.Point(size.width - 1, fiveUp - 2)
                pt2 = wx.Point(size.width - 1, fiveUp + 2)
            else:
                label = str(tick + 1)
                
                rect = wx.Rect(tickAdj * blockWidth, 0, blockWidth, textHeight)
                pt = wxHelper.centerTextPoint(dc, rect, font, label)
                
                pt1 = wx.Point(tickAdj * blockWidth + blockWidth / 2.0, fiveUp - 2)
                pt2 = wx.Point(tickAdj * blockWidth + blockWidth / 2.0, fiveUp + 2)
            
            wxHelper.drawLine(dc, pt1, pt2, color)
            wxHelper.drawTextColor(dc, pt, font, label, color)
            
    def cutTicks(self, start, end, ticks):
        appHelper = getApplicationHelper()
        
        if ticks == 0:
            return []
        
        length = end - start
        divisions = float(length) / (ticks + 1)
        
        tickList = []
        curAt = start + divisions
        for i in range(ticks):
            tickList.append(appHelper.floor(curAt))
            
            curAt += divisions
            
        return tickList
        
    def OnSize(self, event):
        self.Refresh()
        event.Skip()
 
    def setZoom(self, start, end, fromApplication):
        if self.__init and not Zoomable.zoomChanged(self, start, end):
            return
        
        Zoomable.setZoom(self, start, end, fromApplication)

        if self.__init:
            self.Refresh()
            #self.addColumnsIfAppropriate()
            #self.setColumnLabels()
            
class AlnRulerLineSegment(wx.Panel):
    TICK_LEFT = 1
    TICK_RIGHT = 2
    TICK_CENTER = 4
    
    def __init__(self, parent, id, pos = wx.DefaultPosition, size = wx.DefaultSize, style = 0, tickStyle = 0):
        style |= wx.SIMPLE_BORDER
        wx.Panel.__init__(self, parent, id, pos, size, style)
        
        self.setTickStyle(tickStyle)
        
        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_SIZE, self.OnSize)
    
    def OnSize(self, event):    
        self.Refresh()
        
        event.Skip()
        
    def setTickStyle(self, style):
        self.__tickStyle = style
    
    def OnPaint(self, event):
        wxHelper = getWxHelper()
        dc = wx.PaintDC(self)
        
        size = self.GetClientSize()
        black = Color(Color.RGB, 1, 0.7, 0.7)
        
        pt1 = wx.Point(0, size.height / 2)
        pt2 = wx.Point(size.width, size.height / 2)
        
        wxHelper.drawLine(dc, pt1, pt2, black)
        
        if self.__tickStyle & self.TICK_LEFT:
            pt1 = wx.Point(0, size.height / 4)
            pt2 = wx.Point(0, size.height * (3.0 / 4))
            wxHelper.drawLine(dc, pt1, pt2, black)
        if self.__tickStyle & self.TICK_RIGHT:
            pt1 = wx.Point(size.width - 1, size.height / 4)
            pt2 = wx.Point(size.width - 1, size.height * (3.0 / 4))
            wxHelper.drawLine(dc, pt1, pt2, black)    
        if self.__tickStyle & self.TICK_CENTER:
            pt1 = wx.Point(size.width / 2, size.height / 4)
            pt2 = wx.Point(size.width / 2, size.height * (3.0 / 4))
            wxHelper.drawLine(dc, pt1, pt2, black)
            
class AlnRulerLine(wx.Panel):
    def __init__(self, parent, id, pos = wx.DefaultPosition, size = wx.DefaultSize, style = 0):
        wx.Panel.__init__(self, parent, id, pos, size, style)
        
        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_SIZE, self.OnSize)
        
    def OnPaint(self, event):
        dc = wx.PaintDC(self)
        
        aln = getAlignment()
        
        size = self.GetClientSize()
        dc.SetUserScale(size.width, float(size.height) / 6)
        
        pt1 = wx.Point(0, 3)
        pt2 = wx.Point(size.width, 3)
        
        helper = getWxHelper()
        helper.drawLine(dc, pt1, pt2, Color(Color.RGB, 0, 0, 0))
        
    def OnSize(self, event):
        self.Refresh()
        
        event.Skip()