import wx
import wx.glcanvas as glcanvas

from display.applicationhelper import *
from display.wxhelper import *
from display.color import *
from display.window.alnruler import *
from display.window.winutil.canonicalview import *
from display.window.winutil.traditionaltrackdrawer import *
from display.window.winutil.rectangledrawer import *

from display.dialog.addtraditionaltrack import *

from util.alignmentmanager import *
from util.preferences import *

from display.window.winutil.sequencedrawer import *

def makeCDDFeature(name, id, locations = []):
    qualifiers = {'name': name, 'color': id}
    
    if len(locations) == 1:
        instance = CDD(locations[0], 'CDD', '', None, id, qualifiers)
        instance.qualifiers = qualifiers
    else:
        subfeats = []
        
        for loc in locations:
            subfeats.append(CDD(loc, 'CDD', '', None, id, qualifiers))
            
        instance = CDD(None, 'CDD', '', None, id, qualifiers, subfeats)
        instance.sub_features = subfeats
        instance.qualifiers = qualifiers
        
    return instance

class AlnView(CanonicalView, SequenceSwapable):
    def __init__(self, parent, id, pos = wx.DefaultPosition, size = wx.DefaultSize, style = 0, aln = None, zoom = None, seqOrder = None):
        self.__init = False
        CanonicalView.__init__(self, parent, id, pos, size, style, cols = 2, zoom = zoom, displayTitle = aln.getTitle(), aln = aln)
        SequenceSwapable.__init__(self, seqOrder)
        
        # "Add Additional Track" button
        addTrackButton = wx.Button(self, -1, label = "Add Feature Track")
        addTrackButton.Bind(wx.EVT_BUTTON, self.OnAddTrack)
        self.addButton(addTrackButton)
        
        self.__aln = aln
        
        self.__seqLabels = {}
        self.__seqCanvases = {}
        
        # On to the good stuff
        self.__buildAlnDictionary()
        
        for seqIndex in self.seqOrder:          
            self.__seqLabels[seqIndex] = CanonicalLabel(self, -1, label = self.__aln.getSeqDescriptionByIndex(seqIndex), canonicalView = self)
            self.bindDragEvents(self.__seqLabels[seqIndex])
             
            self.__seqCanvases[seqIndex] = AlnViewSequenceCanvas(self, -1, aln = self.__aln, seqIndex = seqIndex, zoom = zoom, alnView = self)
            
            self.addRow(self.__seqLabels[seqIndex], self.__seqCanvases[seqIndex])
        
        featUtils = getFeatureUtilities()
        
        frame = getApplicationFrame()
        
        self.__featureTracks = []
        
        #self.setMinHeightFromRows(range(self.getNumRows()))

        self.SetSizer(self.sizer)
        #self.sizer.Fit(getApplicationFrame())
        self.sizer.Layout()        
        
        # Scrolling on!
        #self.setupScrolling()
        #print self.GetSizeTuple()
        
        self.__init = True
    
    def OnAddTrack(self, event):
        frame = getApplicationFrame()
        wxHelper = getWxHelper()
        
        addTrackDialog = AddTraditionalTrackDialog(self, -1, aln = self.__aln, alnView = self)
        
        try:
            if addTrackDialog.ShowModal() == wx.ID_OK:        
                seqIndex = addTrackDialog.getSequenceSelection()
                if seqIndex == -1:
                    raise Exception()
                
                fetcher = addTrackDialog.getFetcherSelection()
                
                seqRecord = self.__aln.getSeqRecordByIndex(seqIndex)
                
                fetched = self.__aln.getFetcherQueriedState(seqRecord, fetcher)
                if not fetched:
                    wxHelper.displayInfoMessage(self, "The features will now be downloaded from the Internet data.  Please press OK and wait.", "Need to Fetch Features")
                    
                    self.__aln.fetchFeaturesForSeqRecord([fetcher], seqRecord)
                
                labelText = str(self.__aln.getSeqDescriptionByIndex(seqIndex)) + " features"
                label = CanonicalLabel(self, -1, label = labelText, canonicalView = self, bold = True)
                
                track = TraditionalTrackDrawer(self, -1, zoom = (self.start, self.end), seqRecord = seqRecord)
                self.__featureTracks.append(track)
                
                # Expand an arbitrary width down to allow some room
                # TODO: Actually make this work?
                frame = getApplicationFrame()
                #frame.adjustSize(30, 0)    
                
                self.addRow(label, track)
                self.sizer.Layout()
        except:
            wxHelper.displayErrorMessage(frame, "Unable to fetch features!", "Fetcher error")
        finally:
            addTrackDialog.Destroy()
    
    def swapRowOrder(self, one, two):
        self.swapSequenceOrder(one, two)
        
    def swapSequenceOrder(self, one, two, fromApplication = False):
        if not self.__init:
            return
        
        if not SequenceSwapable.orderChanged(self, one, two):
            return
        
        if fromApplication:
            # Only change things if it comes from the application
            SequenceSwapable.swapSequenceOrder(self, one, two, fromApplication)
            CanonicalView.swapRowOrder(self, one, two)
        else:
            # If something screws up, i.e., we're moving a feature track, ignore
            # the error and just switch them
            try:
                frame = getApplicationFrame()
                frame.swapSequenceOrderForAlignment(self.__aln, one, two)
            except:
                CanonicalView.swapRowOrder(self, one, two)
        
    def getCommonLetterByPosition(self, position):
        return self.__commonLetters[position]
    
    def __buildAlnDictionary(self):
        self.__commonLetters = {}
        for i in range(self.__aln.getLength()):
            self.__commonLetters[i] = []
            letterDict = {}
            
            nonGapChars = 0
            column = self.__aln.getColumn(i)
            for letter in column:
                if letter == '-':
                    continue
                
                nonGapChars += 1
                
                if letter not in letterDict:
                    letterDict[letter] = 0
                    
                letterDict[letter] += 1
                
            if nonGapChars >= 1 and nonGapChars <= 2:
                self.__commonLetters[i] = None
            else:
                maxValue = 0
                for letter in letterDict:
                    if letterDict[letter] > maxValue:
                        maxValue = letterDict[letter]
                
                for letter in letterDict:
                    if letterDict[letter] == maxValue:
                        self.__commonLetters[i].append(letter)
                
                if len(self.__commonLetters[i]) == len(column):
                    # If *all* of them are different, just print it all grey
                    self.__commonLetters[i] = []
                        
    def setZoom(self, start, end, fromApplication):
        CanonicalView.setZoom(self, start, end, fromApplication)
        
        if self.__init:
            for canvas in self.__seqCanvases.values():
                canvas.setZoom(start, end, fromApplication)
            
            if start is not None and end is not None:
                for track in self.__featureTracks:
                    track.setZoom(start, end, fromApplication)

class AlnViewSequenceCanvas(SequenceDrawerCanvas):
    def __init__(self, parent, id, pos = wx.DefaultPosition, size = wx.DefaultSize, aln = None, seqIndex = None, zoom = None, alnView = None):
        self.__init = False
        SequenceDrawerCanvas.__init__(self, parent, id, pos, size, aln = aln, drawOnlyIndex = seqIndex, zoom = zoom)
        
        self.__aln = aln
        self.__alnView = alnView
        self.__seqIndex = seqIndex
        
        self.__rectangleDrawer = RectangleDrawer()
        
        # Need to make 2D arrays here, because RectangleTracker is used to 
        # drawing tracks, but we're only drawing one track
        self.__rectangleDrawer.setColors([[
                                            Color(Color.RGB, 0, 0, 0), # Black
                                            Color(Color.RGB, 0.6, 0.6, 0.6), # Dark Grey
                                            Color(Color.RGB, 0.9, 0.9, 0.9), # Light Grey
                                            Color(Color.RGB, 1, 1, 1) # White
                                        ]])
        self.__buildPositionValuesList()
        self.__rectangleDrawer.setPositionValues([self.__positionValues])
        
        #self.setBackgroundColorHints(self.__colors)
        
        self.SetMinSize((self.__aln.getLength(), 13))
        
        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_SIZE, self.OnSize)
        
        self.__init = True
        
    def OnSize(self, event):
        self.Refresh()
        
        event.Skip()
        
    def setZoom(self, start, end, fromApplication = False):
        if self.__init and not Zoomable.zoomChanged(self, start, end):
            return
        
        SequenceDrawerCanvas.setZoom(self, start, end, fromApplication)
        
        if self.__init:
            self.__rectangleDrawer.setZoom(self.start, self.end, fromApplication)
            
            if not fromApplication:
                frame = getApplicationFrame()
                frame.setZoomForAlignment(self.__aln, self.start, self.end)
    
    def OnPaint(self, event):
        self.SetCurrent()
        
        dc = wx.PaintDC(self)
        
        self.setupViewport()

        self.scales = (self.getZoomLength(), 1)
        self.setUserScale(self.scales[0], self.scales[1])
     
        self.clearModelViewMatrix()
        
        self.clearColor()
        
        self.__rectangleDrawer.drawRectangles()
        
        self.drawSequenceStrings()
        
        self.flush()
        
        self.SwapBuffers()
    
    def __buildPositionValuesList(self):
        helper = getApplicationHelper()
        
        self.__positionValues = []
        
        seqString = self.__aln.getSeqStringByIndex(self.__seqIndex)
        
        for i in range(len(seqString)):
            commonLetters = self.__alnView.getCommonLetterByPosition(i)
            letter = seqString[i]
            
            if commonLetters is None and letter != '-':
                self.__positionValues.append(.67)
            else:
                if letter == '-':
                    self.__positionValues.append(1.0) # Gap
                elif letter in commonLetters:
                    self.__positionValues.append(0.0) # Full match
                else:
                    self.__positionValues.append(.33) # Mismatch
                
        # Gap at end of alignment, maybe?
        for i in range(len(seqString), self.__aln.getLength()):
            self.__positionValues.append(1.0)
