import wx

from display.applicationhelper import *
from display.wxhelper import *

from display.window.colorkeyview import *

from display.window.winutil.canonicalview import *

from display.window.featureview.checkboxarray import *

from display.window.winutil.sequencedrawer import *
from display.window.winutil.rectangledrawer import *

from util.preferences import *

class FeatureView(CanonicalView, SequenceSwapable, SequenceNotifiable):
    def __init__(self, parent, id, pos = wx.DefaultPosition, size = wx.DefaultSize, style = 0, aln = None, zoom = None, seqOrder = None):
        self.__init = False        
        CanonicalView.__init__(self, parent, id, pos, size, cols = 2, zoom = zoom, displayTitle = aln.getTitle(), aln = aln)
        SequenceSwapable.__init__(self, seqOrder)
        SequenceNotifiable.__init__(self)
        
        dragLabel = wx.StaticText(self, -1, label = "Drag CD labels to reorder rows")
        self.addToTopLeftCornerSizer(dragLabel, wx.EXPAND | wx.ALIGN_CENTER_VERTICAL)
        hideLabel = wx.StaticText(self, -1, label = "Right-click CD labels to hide")
        self.addToTopLeftCornerSizer(hideLabel, wx.EXPAND | wx.ALIGN_CENTER_VERTICAL)
        
        frame = getApplicationFrame()
        
        self.mainSizer = wx.FlexGridSizer(cols = 1, vgap = 2)
        self.mainSizer.SetFlexibleDirection(wx.BOTH)
        self.mainSizer.AddGrowableCol(0)
        self.mainSizer.AddGrowableRow(4)
        
        self.__aln = aln
        self.__alnLength = self.__aln.getLength()

        self.__featLabels = {}
        self.__featPanels = {}
        
        for featInfoObj in self.__aln.getUniqueFeatureData():
            lowerDescription = ""
            if featInfoObj.evalue >= 0:
                lowerDescription = "Avg. E-Value: %.2e" % (featInfoObj.evalue)
                
            self.__featLabels[featInfoObj.id] = CanonicalLabel(self, -1, label = featInfoObj.name, canonicalView = self, url = featInfoObj.url, lowerDescription = lowerDescription)
            self.bindDragEvents(self.__featLabels[featInfoObj.id])
            
            self.__featPanels[featInfoObj.id] = FeatureViewFeatureCanvas(self, -1, aln = self.__aln, featureID = featInfoObj.id, zoom = zoom, seqOrder = seqOrder)
            
            self.addRow(self.__featLabels[featInfoObj.id], self.__featPanels[featInfoObj.id])
        
        colorKeyView = ColorKeyView(self, -1, aln = self.__aln, seqOrder = seqOrder)
        frame.addWindow(colorKeyView, aln, addToSizer = False)
        self.mainSizer.Add(colorKeyView, 1, wx.EXPAND)
        
        self.mainSizer.Add((0, 4), 1, wx.EXPAND)
        
        self.checkboxArray = FeatureViewCheckboxArray(self, -1, canonicalView = self, aln = self.__aln)
        self.mainSizer.Add(self.checkboxArray, 0, wx.EXPAND)
        
        sepWindow = wx.Window(self, -1)
        sepWindow.SetBackgroundColour(wx.BLACK)
        sepWindow.SetMinSize((0, 2))
        self.mainSizer.Add(sepWindow, 0, wx.EXPAND)
        
        # Hide more than a certain number of features
        preferences = getPreferences()
        notify = False
        for row in range(preferences.getInitialNumberDisplayedFeatures(), self.getNumRows()):
            notify = True
            self.hideRow(row)
            
        if notify:
            # Tell user that we hid some rows
            wxHelper = getWxHelper()
            wxHelper.displayInfoMessage(self, "Some features were hidden to conserve space.  The feature rows currently displayed are the highest quality matches sorted by E-Value.\n\nYou may unhide features by clicking on the \"Unhide All Rows\" button or selecting individual features using the checkboxes above the key")        
        
        #rows = self.getNumRows()
        #if rows > preferences.getInitialNumberDisplayedFeatures():
        #    rows = preferences.getInitialNumberDisplayedFeatures()
        #self.setMinHeightFromRows(range(rows))
        
        self.mainSizer.Add(self.sizer, 1, wx.EXPAND)
        self.SetSizer(self.mainSizer)
        self.mainSizer.Layout()
        #self.mainSizer.Fit(getApplicationFrame())
        
        #self.setupScrolling()
        
        
        #self.Bind(wx.EVT_SIZE, self.OnSize)
        
        self.__init = True
        
    def OnSize(self, event):
        event.Skip()
        
    def getRowFromFeatureID(self, featID):
        return self.getRowFromWindow(self.__featPanels[featID])
    
    def getFeatureIDFromRow(self, row):
        window = self.getWindowFromRow(row)
        
        for featID in self.__featPanels:
            if window == self.__featPanels[featID]:
                return featID
            
        return None
    
    def showRow(self, row, show):
        featID = self.getFeatureIDFromRow(row)
        self.checkboxArray.updateState(featID, show)
                
        CanonicalView.showRow(self, row, show)
        
    def swapSequenceOrder(self, one, two, fromApplication):
        if not self.__init:
            return
        
        if not SequenceSwapable.orderChanged(self, one, two):
            return
        
        SequenceSwapable.swapSequenceOrder(self, one, two, fromApplication)
            
        for panel in self.__featPanels.itervalues():
            panel.swapSequenceOrder(one, two, fromApplication)    
    
    def setZoom(self, start, end, fromApplication):
        CanonicalView.setZoom(self, start, end, fromApplication)
        
        if self.__init:
            for panel in self.__featPanels.itervalues():
                panel.setZoom(start, end, fromApplication)
                
    def setNotifySequence(self, index, fromApplication = False):
        if fromApplication:
            for panel in self.__featPanels.itervalues():
                panel.setNotifySequence(index, fromApplication)
            

class FeatureViewFeatureCanvas(SequenceDrawerCanvas, SequenceNotifiable):
    def __init__(self, parent, id, pos = wx.DefaultPosition, size = wx.DefaultSize, zoom = None, aln = None, featureID = None, seqOrder = None):
        self.__init = False
        SequenceDrawerCanvas.__init__(self, parent, id, pos, size, zoom = zoom, aln = aln, seqOrder = seqOrder)
        SequenceNotifiable.__init__(self)
        
        appHelper = getApplicationHelper()
        
        self.__aln = aln
        self.__alnLength = self.__aln.getLength()
        self.__numSeqs = self.__aln.getNumSequences()
        self.__featureID = featureID
        
        print "Order: %s" % (str(seqOrder))
        self.__rectangleDrawer = RectangleDrawer(order = seqOrder, drawBorder = True)
        
        colorKey = getColorKey(self.__aln)
        
        (colors, posValues) = self.__buildDicts()
        self.__rectangleDrawer.setColors(colors)
        self.__rectangleDrawer.setAltColors(colorKey.getBackgroundBase())
        self.__rectangleDrawer.setPositionValues(posValues)
        self.__rectangleDrawer.setRefreshCallback(self.Refresh)
        
        self.SetMinSize((self.__alnLength, 13 * self.__numSeqs))
        
        self.Bind(wx.EVT_PAINT, self.OnPaint)
        #self.Bind(wx.EVT_SIZE, self.OnSize)
       
        self.__init = True
        
    def setNotifySequence(self, index, fromApplication = False):
        if fromApplication:
            self.__rectangleDrawer.setNotifySequence(index, fromApplication)
        
    def OnSize(self, event):
        self.Refresh()
        
        event.Skip()
        
    def OnPaint(self, event):
        self.SetCurrent()
        
        dc = wx.PaintDC(self)
        
        self.setupViewport()
     
        #self.clearModelViewMatrix()
        self.clearColor()
        
        self.scales = (self.getZoomLength(), 1)
        self.setUserScale(self.scales[0], self.scales[1])
        
        self.__rectangleDrawer.drawRectangles()
        
        self.drawSequenceStrings()
        
        self.flush()
        
        self.SwapBuffers()        
        
    def setZoom(self, start, end, fromApplication):
        if self.__init and not Zoomable.zoomChanged(self, start, end):
            return
        
        SequenceDrawerCanvas.setZoom(self, start, end, fromApplication)
        
        if self.__init:
            self.__rectangleDrawer.setZoom(self.start, self.end, fromApplication)
            
            if not fromApplication:
                frame = getApplicationFrame()
                frame.setZoomForAlignment(self.__aln, self.start, self.end)        
            
            self.Refresh()
            
    def swapSequenceOrder(self, one, two, fromApplication):
        if not self.__init:
            return
        
        if not SequenceSwapable.orderChanged(self, one, two):
            return
        
        SequenceDrawerCanvas.swapSequenceOrder(self, one, two, fromApplication) 
        self.__rectangleDrawer.swapRowOrder(one, two)
        
        #colorKey = getColorKey(self.__aln)
        
        self.Refresh()
       
    def __buildDicts(self):
        colorKey = getColorKey(self.__aln)
        appHelper = getApplicationHelper()
        
        colors = {}
        posValues = {}
        
        for i in range(self.__numSeqs):
            featureInTrack = self.__aln.getFeatureFromIndexAndID(i, self.__featureID)
            
            colors[i] = colorKey.get(i)
            
            posValues[i] = []
            for j in range(self.__alnLength):
                score = self.__aln.getColumnScore(j)
                atLocation = (featureInTrack is not None and featureInTrack.determineAtLocation(j) & DrawableFeature.LOCATION_AT)
                
                # Since there is no negative zero, a bit of a hack here
                if score == 0:
                    score = 0.001
                
                if atLocation:
                    posValues[i].append(score)
                else:
                    posValues[i].append(-score)
        
        return (colors, posValues)
