import wx

from display.applicationhelper import *
from display.wxhelper import *

class FeatureViewCheckboxArray(wx.Panel, Filterable):
    EVALUE = 1
    NUMSEQS = 2
    
    def __init__(self, parent, id, pos = wx.DefaultPosition, size = wx.DefaultSize, style = 0, canonicalView = None, aln = None):
        wx.Panel.__init__(self, parent, id, pos, size, style)
        Filterable.__init__(self)
        
        self.__canonicalView = canonicalView
        self.__aln = aln
        
        self.__sizer = wx.FlexGridSizer(cols = 5, vgap = 2, hgap = 2)
        self.__sizer.SetFlexibleDirection(wx.VERTICAL)
        
        self.__checkboxArray = {}
        self.__evalues = {}
        self.__numSeqs = {}
        for featInfoObj in self.__aln.getUniqueFeatureData():
            self.__evalues[featInfoObj.id] = featInfoObj.evalue
            self.__numSeqs[featInfoObj.id] = featInfoObj.numberPresent
            
            label = "%s (%d seqs, %.3e)" % (featInfoObj.name, featInfoObj.numberPresent, featInfoObj.evalue)
            self.__checkboxArray[featInfoObj.id] = wx.CheckBox(self, -1, label = label)
            self.__checkboxArray[featInfoObj.id].SetValue(True)
            
            # Bold font
            font = self.__checkboxArray[featInfoObj.id].GetFont()
            font.SetWeight(wx.FONTWEIGHT_BOLD)
            self.__checkboxArray[featInfoObj.id].SetFont(font)
            
            self.__checkboxArray[featInfoObj.id].Bind(wx.EVT_CHECKBOX, self.OnCheckbox)
            
            self.__sizer.Add(self.__checkboxArray[featInfoObj.id], 1, wx.EXPAND | wx.ALIGN_CENTER)
            
        self.__mainSizer = wx.BoxSizer(wx.VERTICAL)
        
        self.__filterSizer = wx.FlexGridSizer(rows = 2, vgap = 10, hgap = 0)
        
        wxHelper = getWxHelper()
        
        minEvalue, maxEvalue = (min(self.__evalues.values()), max(self.__evalues.values()))
        minSeqs, maxSeqs = (min(self.__numSeqs.values()), max(self.__numSeqs.values()))
        
        self.__filterSizer.Add(wxHelper.makeBoldStaticText(self, -1, "E-Value Filter", style = wx.ALIGN_CENTER), 1, wx.EXPAND | wx.ALIGN_CENTER)
        self.__filterSizer.AddGrowableCol(0)
        
        self.__filterSizer.Add(wxHelper.makeBoldStaticText(self, -1, "Number Seqs. Present Filter", style = wx.ALIGN_CENTER), 1, wx.EXPAND | wx.ALIGN_CENTER)
        self.__filterSizer.AddGrowableCol(1)
        
        self.evalueSlider = RangeSlider(self, -1, value = minEvalue, minValue = minEvalue, maxValue = maxEvalue, useLogs = True)
        self.evalueSlider.Bind(wx.EVT_SCROLL_CHANGED, self.OnEvalueSliderChange)
        self.setFilterRange(self.EVALUE, minEvalue, maxEvalue)
        self.__filterSizer.Add(self.evalueSlider, 1, wx.EXPAND)
        
        self.numSeqsSlider = RangeSlider(self, -1, value = minSeqs, minValue = minSeqs, maxValue = maxSeqs, useLogs = False, labelFormat = "%d", segments = (maxSeqs - minSeqs))
        self.numSeqsSlider.Bind(wx.EVT_SCROLL_CHANGED, self.OnNumSeqsSliderChange)
        self.setFilterRange(self.NUMSEQS, minSeqs, maxSeqs)
        self.__filterSizer.Add(self.numSeqsSlider, 1, wx.EXPAND)
        
        self.__mainSizer.Add(self.__filterSizer, 0, wx.EXPAND)
        self.__mainSizer.Add(wx.StaticText(self, -1, label = "Feature Name (Number of Sequences, E Value)"), 0, wx.EXPAND)            
        self.__mainSizer.Add(self.__sizer, 0, wx.EXPAND)
            
        self.SetSizer(self.__mainSizer)
    
    def OnEvalueSliderChange(self, event):
        self.setFilterRange(self.EVALUE, self.evalueSlider.getFloatSelStart(), self.evalueSlider.getFloatSelEnd())
        
        self.updateVisibleStates()
        
        print "Change!: %e" % (self.evalueSlider.getFloatValue())
        
    def OnNumSeqsSliderChange(self, event):
        self.setFilterRange(self.NUMSEQS, self.numSeqsSlider.getFloatSelStart(), self.numSeqsSlider.getFloatSelEnd())
        
        self.updateVisibleStates()
        
        print "Change!: %d" % (self.numSeqsSlider.getFloatValue())
    
    def updateVisibleStates(self):
        for featID in self.__checkboxArray:
            evalue = self.__evalues[featID]
            numSeqs = self.__numSeqs[featID]
            
            # TODO: and OR or here?
            if self.inFilterRange(self.EVALUE, evalue) and self.inFilterRange(self.NUMSEQS, numSeqs):
                self.__sizer.Show(self.__checkboxArray[featID], True)
            else:
                self.__sizer.Show(self.__checkboxArray[featID], False)
                
                if self.__checkboxArray[featID].GetValue():
                    self.updateState(featID, False)
                    self.showFeatureRow(featID, False)
        
        # TODO: How much of this is necessary?
        self.__filterSizer.Layout()    
        self.__sizer.Layout()
        self.__mainSizer.Layout()
        
        self.__canonicalView.refreshView()
        
        self.Refresh()
        
        frame = getApplicationFrame()
        frame.doLayout()
    
    def updateState(self, featID, state):
        if featID in self.__checkboxArray:
            self.__checkboxArray[featID].SetValue(state)
        
    def OnCheckbox(self, event):
        featID = None
        for id in self.__checkboxArray:
            if self.__checkboxArray[id] == event.GetEventObject():
                featID = id
                break
        
        if featID is not None:
            self.showFeatureRow(featID, event.IsChecked())
        
    def showFeatureRow(self, featID, show):        
        row = self.__canonicalView.getRowFromFeatureID(featID)
        if row is None:
            return
        
        self.__canonicalView.showRow(row, show)
        
class FeatureViewCheckboxArrayFilterPanel(wx.Panel):
    def __init__(self, parent, id, pos = wx.DefaultPosition, size = wx.DefaultSize, style = 0, checkboxArray = None, aln = None):
        wx.Panel.__init__(self, parent, id, pos, size, style)

        self.__checkboxArray = checkboxArray
        self.__aln = aln
        
        