import wx
import wx.lib.scrolledpanel as scrolledpanel

from display.application import *
from display.applicationhelper import *
from display.window.alnruler import *
from display.wxhelper import *
from display.dialog.zoomtoregion import *

class CanonicalView(wx.Panel, Zoomable):
    def __init__(self, parent, id, pos = wx.DefaultPosition, size = wx.DefaultSize, style = 0, cols = 2, zoom = None, buttonBar = True, alnRuler = True, displayTitle = None, vgap = 2, hgap = 3, aln = None, scrolling = True):
        self.__init = False
        wx.Panel.__init__(self, parent, id, pos, size, style)
        Zoomable.__init__(self, zoom)
        
        self.topLeftCornerSizer = wx.BoxSizer(wx.VERTICAL)
        
        self.__aln = aln
            
        #self.sizer = wx.FlexGridSizer(cols = 1)
        #self.sizer.AddGrowableCol(0)
        self.sizer = wx.BoxSizer(wx.VERTICAL)            
            
        self.informationWindow = CanonicalViewInformationWindow(self, -1, canonicalView = self)
        
        if displayTitle:
            self.titleBar = CanonicalViewTitleBar(self, -1, label = displayTitle)
            
            self.informationWindow.addRow(self.titleBar)
        
        infoSizer = wx.FlexGridSizer(rows = 1)
        infoSizer.AddGrowableRow(0)
        
        infoSizer.Add(self.topLeftCornerSizer, 1, wx.EXPAND)
        
        if buttonBar:
            self.buttonBar = CanonicalViewButtonBar(self, -1)
            
            # Buttons
            unhideAllButton = wx.Button(self, -1, label = 'Unhide All Rows')
            unhideAllButton.Bind(wx.EVT_BUTTON, self.OnUnhideButton)
            self.addButton(unhideAllButton)
            
            if self.__aln is not None:
                zoomRegion = wx.Button(self, -1, label = 'Zoom To Region')
                zoomRegion.Bind(wx.EVT_BUTTON, self.OnZoomRegionButton)
                self.addButton(zoomRegion)
            
            zoomoutButton = wx.Button(self, -1, label = 'Zoom Out To Full Length')
            zoomoutButton.Bind(wx.EVT_BUTTON, self.OnZoomoutButton)
            self.addButton(zoomoutButton)
            
            refreshButton = wx.Button(self, -1, label = 'Refresh View')
            refreshButton.Bind(wx.EVT_BUTTON, self.OnRefreshView)
            self.addButton(refreshButton)
            
            removeButton = wx.Button(self, -1, label = 'Remove This View')
            removeButton.Bind(wx.EVT_BUTTON, self.OnRemoveThisViewButton)
            self.addButton(removeButton)
            
            infoSizer.Add(self.buttonBar, 1, wx.EXPAND)
            
        self.informationWindow.addRow(infoSizer)

        self.rowsWindow = CanonicalViewRowsWindow(self, -1, canonicalView = self, cols = cols, vgap = vgap, hgap = hgap, scrolling = scrolling)       
        
        self.alnRuler = None
        if alnRuler:
            self.alnRuler = AlnRuler(self, -1, zoom = zoom)
            self.rowsWindow.setRuler(self.alnRuler)
        
        self.sizer.Add(self.informationWindow, 0, wx.EXPAND)
        self.sizer.Add(self.rowsWindow, 1, wx.EXPAND)
        
        self.__dragLabel = None
        
        #self.sizer.Layout()
        #self.SetSizer(self.sizer)
        
        self.Bind(wx.EVT_SIZE, self.OnSize)
        
        self.__init = True
    
    def OnSize(self, event):
        self.doLayout()
        
        event.Skip()
    
    def OnZoomRegionButton(self, event):
        zoomRegionDialog = ZoomToRegionDialog(self, -1, aln = self.__aln)
        
        zoomRegionDialog.ShowModal()
        
        self.setFocusToRowOne()
        
    def OnRemoveThisViewButton(self, event):
        frame = getApplicationFrame()
        frame.removeWindow(self, True)
    
    def addToTopLeftCornerSizer(self, window, flags = wx.EXPAND):
        self.topLeftCornerSizer.Add(window, 0, flags)
        self.topLeftCornerSizer.Layout()
    
    def OnRefreshView(self, event):
        self.refreshView()
        
    def refreshView(self, notifyFrame = True):
        self.doLayout()
        
        self.rowsWindow.Refresh()
        self.informationWindow.Refresh()
        
        if notifyFrame:
            frame = getApplicationFrame()
            frame.doLayout()
            frame.Refresh()
    
    def doLayout(self):
        self.sizer.Layout()
        self.Refresh()
    
    def OnZoomoutButton(self, event):
        self.setZoom(None, None, False)
        
        self.setFocusToRowOne()
    
    def OnUnhideButton(self, event):
        for i in range(self.getNumRows()):
            self.unhideRow(i)
            
        self.setFocusToRowOne()
    
    def swapRowOrder(self, one, two):
        self.rowsWindow.swapRowOrder(one, two)
        
    def setZoom(self, start, end, fromApplication = False):
        if self.__init and not Zoomable.zoomChanged(self, start, end):
            return
            
        Zoomable.setZoom(self, start, end, fromApplication)
        
        if self.__init:
            if self.alnRuler is not None:
                self.alnRuler.setZoom(start, end, fromApplication)
       
    def bindDragEvents(self, window):
        self.rowsWindow.bindDragEvents(window)       
        
    def getNumRows(self):
        return self.rowsWindow.getNumRows()
            
    def addRow(self, *cols):
        self.rowsWindow.addRow(*cols)
    
    def getRowFromPosition(self, pos):
        return self.rowsWindow.getRowFromPosition(pos)
    
    def getPositionFromRow(self, row):
        return self.rowsWindow.getPositionFromRow(row)
    
    def getRowFromWindow(self, window):
        return self.rowsWindow.getRowFromWindow(window)
        
    def getWindowFromPosition(self, position):
        return self.rowsWindow.getWindowFromPosition(position)
        
    def getWindowFromRow(self, row):
        return self.rowsWindow.getWindowFromRow(row)
    
    def getNumCols(self):
        return self.rowsWindow.getNumCols()
        
    def showRow(self, row, show):
        self.rowsWindow.showRow(row, show)
                
    def hideRow(self, row):
        self.showRow(row, False)
        
    def unhideRow(self, row):
        self.showRow(row, True)
        
    def addButton(self, button):
        self.buttonBar.addButton(button)
        
        self.sizer.Layout()
        
    def setupScrolling(self):
        self.rowsWindow.setupScrolling()
        
    def setMinHeightFromRows(self, *args):
        self.rowsWindow.setMinHeightFromRows(*args)
        
    def getVisibleRows(self):
        return self.rowsWindows.getVisibleRows()
    
    def setMinHeightFromVisibleRows(self):
        return self.rowsWindows.setMinHeightFromVisibleRows()
        
    def setFocusToRowOne(self):
        self.rowsWindow.setFocusToRowOne()        
        
class CanonicalLabel(ContextMenuableWxPanel):
    HIDE = 1001
    
    def __init__(self, parent, id, pos = wx.DefaultPosition, size = wx.DefaultSize, style = 0, label = "", canonicalView = None, bold = False, hideMenu = True, url = None, lowerDescription = "", bgColor = None):
        ContextMenuableWxPanel.__init__(self, parent, id, pos, size, style)
        
        if bgColor is None:
            bgColor = Color(Color.RGB, 0.75, 0.75, 0.75)
        bgColor.provideToFunctionAsWxColour(self.SetBackgroundColour)
        
        self.__canonicalView = canonicalView
        
        self.__sizer = wx.BoxSizer(wx.VERTICAL)
        
        if url is None:
            if bold:
                wxHelper = getWxHelper()
                self.label = wxHelper.makeBoldStaticText(self, -1, label = label)
            else:
                self.label = wx.StaticText(self, -1, label = label)
        else:
            self.label = wx.HyperlinkCtrl(self, -1, label = label, url = url)
            self.label.SetVisitedColour(self.label.GetNormalColour())
            
        if hideMenu:
            menu = wx.Menu()
            menu.Append(CanonicalLabel.HIDE, 'Hide Row', 'Hide Row')
            
            self.Bind(wx.EVT_MENU, self.OnMenuHide, id = CanonicalLabel.HIDE)
            
            self.setContextMenu(menu)
        
        self.label.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
        self.label.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
        self.label.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDown)
        
        labelStyle = wx.ALIGN_CENTER_VERTICAL
        if lowerDescription:
            labelStyle = wx.ALIGN_BOTTOM 
        
        self.__sizer.Add(self.label, 1, wx.ALIGN_CENTER | wx.EXPAND | labelStyle)
        
        if lowerDescription:
            self.lowerLabel = wx.StaticText(self, -1, label = lowerDescription)
            
            self.lowerLabel.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
            self.lowerLabel.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
            self.lowerLabel.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDown)
            
            self.__sizer.Add(self.lowerLabel, 1, wx.ALIGN_CENTER | wx.ALIGN_TOP | wx.EXPAND)        
        
        self.SetSizer(self.__sizer)
    
    def OnMenuHide(self, event):
        if self.__canonicalView is not None:
            row = self.__canonicalView.getRowFromWindow(self)
            
            if row is not None:
                self.__canonicalView.hideRow(row)
    
    def Refresh(self):
        wx.Panel.Refresh(self)
        self.label.Refresh()
    
    def OnLeftDown(self, event):
        event.SetEventObject(self)
        
        self.ProcessEvent(event)
        event.Skip()
        
    def OnLeftUp(self, event):
        event.SetEventObject(self)
        
        self.ProcessEvent(event)
        event.Skip()
        
class CanonicalViewButtonBar(wx.Panel):
    def __init__(self, parent, id, pos = wx.DefaultPosition, size = wx.DefaultSize, style = 0):
        wx.Panel.__init__(self, parent, id, pos, size, style)
        
        self.__sizer = wx.GridSizer(cols = 6, hgap = 1, vgap = 1)
        
        self.SetSizer(self.__sizer)
        
    def addButton(self, button):
        button.Reparent(self)
        self.__sizer.Add(button, 1, wx.EXPAND)
        
        self.__sizer.Layout()

class CanonicalViewTitleBar(wx.Panel):
    def __init__(self, parent, id, pos = wx.DefaultPosition, size = wx.DefaultSize, style = 0, label = ""):
        wx.Panel.__init__(self, parent, id, pos, size, style)
    
        self.__sizer = wx.BoxSizer(wx.HORIZONTAL)
    
        wxHelper = getWxHelper()
        self.__label = wxHelper.makeBoldStaticText(self, -1, label)
        self.__sizer.Add(self.__label, 1, wx.EXPAND | wx.ALIGN_CENTER)
        
        self.SetSizer(self.__sizer)
        
class CanonicalViewInformationWindow(wx.Panel):
    def __init__(self, parent, id, pos = wx.DefaultPosition, size = wx.DefaultSize, style = 0, canonicalView = None):
        wx.Panel.__init__(self, parent, id, pos, size, style)
        
        self.__canonicalView = canonicalView
        
        self.__sizer = wx.BoxSizer(wx.VERTICAL)
        
        self.SetSizer(self.__sizer)
        
    def addRow(self, win):
        if isinstance(win, wx.Window):
            win.Reparent(self)
        elif isinstance(win, wx.Sizer):
            for sizerItem in win.GetChildren():
                sizerWin = sizerItem.GetWindow()
                
                if sizerWin is not None:
                    sizerWin.Reparent(self)
        
        self.__sizer.Add(win, 0, wx.EXPAND)
        
        self.__sizer.Layout()

class CanonicalViewRowsWindow(wx.Panel):
    def __init__(self, parent, id, pos = wx.DefaultPosition, size = wx.DefaultSize, style = 0, cols = 2, canonicalView = None, vgap = 2, hgap = 2, scrolling = False, bgColor = None):
        self.__init = False
        wx.Panel.__init__(self, parent, id, pos, size, style)

        self.__canonicalView = canonicalView

        #self.__scrolling = scrolling
        self.__scrolling = False

        self.__ruler = None

        self.__cols = cols
        
        self.__hgap = hgap
        self.__vgap = vgap
        
        self.__visibleRows = []
        
        self.sizer = wx.FlexGridSizer(cols = cols, vgap = vgap, hgap = hgap)
        self.sizer.SetFlexibleDirection(wx.BOTH)
        #self.sizer = wx.BoxSizer(wx.VERTICAL)
        
        # Ruler row
        for i in range(cols):
            self.sizer.Add((0, 0), 0)
        
        self.sizer.AddGrowableCol(cols - 1)
        
        self.__dragLabel = None
     
        if bgColor is None:
            bgColor = Color(Color.RGB, 0.4, 0.4, 0.4)
        bgColor.provideToFunctionAsWxColour(self.SetBackgroundColour)
     
        self.SetSizer(self.sizer)
        
        #self.setupScrolling()        
        
        self.Bind(wx.EVT_SIZE, self.OnSize)
        self.Bind(wx.EVT_SCROLL, self.OnScroll)
        self.Bind(wx.EVT_SCROLLWIN, self.OnScroll)
        
        self.__init = True
             
    def setMinHeightFromVisibleRows(self, adjustCurSize = False):
        return self.setMinHeightFromRows(self.getVisibleRows(), adjustCurSize)
             
    def setMinHeightFromRows(self, rows, adjustCurSize = False):
        height = self.__vgap * (len(rows) + 2)
        
        if isinstance(self.__ruler, wx.Window):
            height += self.__ruler.GetMinSize().height
        
        for row in rows:
            windows = self.getWindowsFromRow(row)
            
            rowHeight = 0
            for win in windows:
                if isinstance(win, wx.Window):
                    winHeight = win.GetMinSize().height
                    
                    if winHeight > rowHeight:
                        rowHeight = winHeight
                        
            height += rowHeight
            
        self.SetMinSize((self.GetMinSize().width, height))
        
        if adjustCurSize:
            curSize = self.GetSize()
            if curSize.height > height:
                self.SetSize((curSize.width, height))
        
        self.refreshAllWindows()
        #self.setupScrolling()
        
        return height
             
    def OnSize(self, event):
        self.sizer.Layout()
        
        event.Skip()
             
    def setRuler(self, ruler):
        if isinstance(ruler, wx.Window):
            ruler.Reparent(self)
        
        self.__ruler = ruler
        
        self.sizer.Detach(self.__cols - 1)
        self.sizer.Insert(self.__cols - 1, ruler, 0, wx.EXPAND)
        
        self.sizer.Layout()
        
    def addRow(self, *cols):
        i = 0
        for col in cols:
            if isinstance(col, wx.Window):
                col.Reparent(self)                    
                
            self.sizer.Add(col, 1, wx.EXPAND | wx.ALIGN_CENTER_VERTICAL)
            
            i += 1
            
        # Not minus one here because we have the ruler row
        #self.sizer.AddGrowableRow(self.getNumRows())
        self.sizer.Layout()
        
        self.__visibleRows.append(self.getNumRows() - 1)
        
        self.setMinHeightFromVisibleRows()
        
        self.setFocusToRowOne()
        
    def insertRow(self, before, *cols):
        i = 0
        for col in cols:
            self.sizer.Insert(before + i, col, 1, wx.EXPAND | wx.ALIGN_CENTER_VERTICAL)
            i += 1
            
    def bindDragEvents(self, window):
        if isinstance(window, wx.Window):
            window.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
            window.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
            
    def OnLeftDown(self, event):    
        self.__dragLabel = event.GetEventObject()
    
    def OnLeftUp(self, event):
        obj = event.GetEventObject()
        if self.__dragLabel is None or obj is None:
            return
        
        wxHelper = getWxHelper()
        position = wxHelper.getEventAbsolutePosition(event)
        
        dragLabelIndex = None
        dropLabelIndex = None
        
        # Cycle through all labels
        for row in range(self.getNumRows()):
            windows = self.getWindowsFromRow(row)
            
            for win in windows:
                # Try to find the ones we want to look at
                if win == obj:
                    dropLabelIndex = row
                    break
                if win == self.__dragLabel:
                    dragLabelIndex = row
                    break
        
        if dropLabelIndex is not None and dragLabelIndex is not None:
            self.swapRowOrder(dragLabelIndex, dropLabelIndex)
            
    def getRowFromPosition(self, pos):
        # -1 for ruler row            
        return pos / self.__cols - 1
    
    def getPositionFromRow(self, row):
        # +1 for ruler row
        return (row + 1) * self.__cols
    
    def getAllWindows(self):
        windows = []
        for sizerItem in self.sizer.GetChildren():
            win = sizerItem.GetWindow()
            if win is not None:
                windows.append(win)
                
        return windows
    
    def getRowFromWindow(self, window):
        for i in range(self.getNumRows()):
            for win in self.getWindowsFromRow(i):
                if win == window:
                    return i
                
        return None
    
    def getNumRows(self):
        return (len(self.sizer.GetChildren()) / self.__cols) - 1
    
    def getNumCols(self):
        return self.__cols
        
    def getWindowFromPosition(self, position):
        sizerItem = self.sizer.GetItem(position)
        return sizerItem.GetWindow()
        
    def getWindowFromRow(self, row):
        position = self.getPositionFromRow(row)

        return self.getWindowFromPosition(position + (self.__cols - 1))
    
    def showRow(self, row, show):
        if show and row not in self.__visibleRows:
            self.__visibleRows.append(row)
        elif not show and row in self.__visibleRows:
            self.__visibleRows.remove(row)
        
        # Inverse of getNumRows formula
        realStart = self.getPositionFromRow(row)
        
        # Hide all columns in the row
        for i in range(self.__cols):
            self.sizer.Show(realStart + i, show)
        
        self.__canonicalView.refreshView()
        
        self.setMinHeightFromVisibleRows()
        self.setFocusToRowOne()
                
    def hideRow(self, row):
        self.showRow(row, False)
        
    def unhideRow(self, row):
        self.showRow(row, True)
    
    def getVisibleRows(self):
        return self.__visibleRows
    
    def getWindowsFromRow(self, row):
        position = self.getPositionFromRow(row)
        
        re = []
        for i in range(self.__cols):
            re.append(self.getWindowFromPosition(position + i))
            
        return re
    
    def swapRowOrder(self, one, two):
        if not self.__init:
            return
        
        if one == two:
            return

        newOne = min(one, two)
        newTwo = max(two, one)
        
        # Get positions
        onePos = self.getPositionFromRow(newOne)
        twoPos = self.getPositionFromRow(newTwo)
        
        # Use rows here, not positions
        oneWindows = self.getWindowsFromRow(newOne)
        twoWindows = self.getWindowsFromRow(newTwo)
        
        # Detach everything in the row
        for win in twoWindows:
            self.sizer.Detach(win)
        for win in oneWindows:
            self.sizer.Detach(win)
            
        # Insert everything back
        self.insertRow(onePos, *twoWindows)
        self.insertRow(twoPos, *oneWindows)
        
        self.setFocusToRowOne()
        
        self.refreshAllWindows()
            
    def setupScrolling(self):
        self.sizer.Layout()
        
        if self.__scrolling:
            self.SetupScrolling(False, True)
            
    def refreshAllWindows(self, layout = True, canonicalRefresh = True):
        if self.__init:
            if layout:
                self.sizer.Layout()
            if canonicalRefresh:
                self.__canonicalView.refreshView()
        
            for win in self.getAllWindows():
                if isinstance(win, wx.Window):
                    win.Refresh()
                    
    def OnScroll(self, event):
        print "Scrolling!"
        print event
        print dir(event)
        print event.__dict__
        self.refreshAllWindows(False, True)
        #self.Refresh()
        
        event.Skip()
        
    def setFocusToRowOne(self):
        if self.getNumRows() >= 1:
            oneWindow = self.getWindowFromRow(0)
            oneWindow.SetFocus()