from display.applicationhelper import *
from display.openglhelper import *
from display.wxhelper import *

class InteractiveZoomableGLPanel(GLPanel, Zoomable):
    def __init__(self, parent, id, pos = wx.DefaultPosition, size = wx.DefaultSize, zoom = None):
        self.__init = False
        GLPanel.__init__(self, parent, id, pos, size)        
        Zoomable.__init__(self, zoom)
        
        self.Bind(wx.EVT_MOTION, self.OnMotion)
        self.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
        self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
        self.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDown)
        self.Bind(wx.EVT_MOUSEWHEEL, self.OnMouseWheel)
        self.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)        
        
        self.__lastPosition = None
        self.__startCoords = None
        
        self.__init = True
                
    def OnKeyDown(self, event):
        code = event.GetKeyCode()
            
        if code == wx.WXK_RIGHT or code == wx.WXK_NUMPAD_RIGHT or code == wx.WXK_WINDOWS_RIGHT:
            self.moveZoom(+1)
        elif code == wx.WXK_LEFT or code == wx.WXK_NUMPAD_LEFT or code == wx.WXK_WINDOWS_LEFT:
            self.moveZoom(-1)
        elif code == wx.WXK_HOME:
            self.setZoom(0, self.end - self.start)
        elif code == wx.WXK_END:
            # This isn't possible because we don't know the absolute end :-(
            pass
                
    def setZoom(self, start, end, fromApplication):
        Zoomable.setZoom(self, start, end, fromApplication)
        
        if self.__init:
            self.Refresh()
                
    def moveZoom(self, delta):
        self.setZoom(self.start + delta, self.end + delta, False)
                
    def OnLeftDown(self, event):
        helper = getWxHelper()
        self.__startCoords = event.GetPosition()
        self.__lastPosition = None
        
        if not self.HasCapture():
            self.CaptureMouse()
        
        event.Skip()    
    
    def OnLeftUp(self, event):
        self.__drawZoomRectangle(self.__lastPosition)
        
        self.__handleZoom(self.__startCoords, self.__lastPosition)
        
        self.__startCoords = None
        
        if self.HasCapture():
            self.ReleaseMouse()
        
        event.Skip()
        
    def __handleZoom(self, startCoords, endCoords):
        if startCoords is None or endCoords is None:
            return
        
        self.SetCurrent()
        
        openGLHelper = getOpenGLHelper()
        appHelper = getApplicationHelper()
        
        startX = startCoords.x
        endX = endCoords.x
        
        if startX > endX:
            startX = endCoords.x
            endX = startCoords.x         
        
        try:
            startPosition = openGLHelper.unproject(startX, 0)
            startPosition = self.start + appHelper.floor(startPosition[0])
            
            endPosition = openGLHelper.unproject(endX, 0)
            endPosition = self.start + appHelper.ceil(endPosition[0])
        except:
            return
        
        if startPosition == endPosition:
            # Can't zoom that close!
            return
        
        self.setZoom(startPosition, endPosition, False)
    
    def __drawZoomRectangle(self, curPos):
        helper = getWxHelper()
        dc = wx.ClientDC(self)
        dc.SetLogicalFunction(wx.INVERT)

        dc.BeginDrawing()

        if self.__startCoords is None or curPos is None:
            dc.EndDrawing()
            return
        
        rect = wx.Rect(self.__startCoords.x, self.__startCoords.y, curPos.x - self.__startCoords.x, curPos.y - self.__startCoords.y)
        color = Color(Color.RGB, 0, 0, 0)

        helper.drawUnfilledRectangle(dc, rect, color)
        dc.EndDrawing()
      
    def OnMouseWheel(self, event):
        if event.GetWheelRotation() < 0:
            self.moveZoom(+1)
        else:
            self.moveZoom(-1)
        
    def OnMotion(self, event):
        if not event.Dragging() or not event.LeftIsDown():
            return
        
        if self.__lastPosition is not None:
            self.__drawZoomRectangle(self.__lastPosition)
        
        helper = getWxHelper()
        
        size = self.GetClientSize()
        position = helper.limitCoordsToSize(event.GetPosition(), size)
        
        self.__drawZoomRectangle(position)
        self.__lastPosition = position
        
        #event.Skip()
        
    def OnRightDown(self, event):
        self.setZoom(None, None, False)
        
        #event.Skip()
    