from display.window.winutil.interactivezoomable import *

from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *

from util.preferences import *

from display.applicationhelper import *
from display.wxhelper import *
from display.openglhelper import *
from display.color import *

class SequenceDrawerCanvas(InteractiveZoomableGLPanel, SequenceSwapable):
    def __init__(self, parent, id, pos = wx.DefaultPosition, size = wx.DefaultSize, zoom = None, aln = None, seqOrder = None, drawOnlyIndex = -1):
        self.__init = False
        InteractiveZoomableGLPanel.__init__(self, parent, id, pos, size, zoom = zoom)
        SequenceSwapable.__init__(self, seqOrder)
        
        self.__aln = aln
        
        self.__drawOnlyIndex = drawOnlyIndex
        if self.__drawOnlyIndex == -1:
            self.__alnLength = aln.getLength()
            self.__numSeqs = aln.getNumSequences()
        else:
            seq = self.__aln.getSeqStringByIndex(self.__drawOnlyIndex)
            self.__alnLength = len(seq)
            self.__numSeqs = 1
        
        self.SetMinSize((self.__alnLength, 20 * self.__numSeqs))
        
        self.scales = (1, 1)
        
        self.__init = True
        
    def setZoom(self, start, end, fromApplication = False):
        if self.__init:
            if start is None and end is None:
                start = 0
                end = self.__alnLength
            elif start < 0:
                return
            elif end > self.__alnLength:
                return
        
            if not Zoomable.zoomChanged(self, start, end):
                return
            
        InteractiveZoomableGLPanel.setZoom(self, start, end, fromApplication)
    
    def swapSequenceOrder(self, one, two, fromApplication):
        if not self.__init:
            return
        
        if not SequenceSwapable.orderChanged(self, one, two):
            return
        
        if fromApplication:
            SequenceSwapable.swapSequenceOrder(self, one, two, fromApplication)
            
    def drawSequenceStrings(self):
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        glEnable(GL_BLEND);
        glEnable(GL_LINE_SMOOTH);
        
        appHelper = getApplicationHelper()
        
        size = self.GetClientSize()
        length = self.getZoomLength()
        
        preferences = getPreferences()
        if (size.width / length) < (preferences.getMinimumDrawableCharacterWidth()):
            return
        
        # Save the matrices with glPushMatrix
        glMatrixMode(GL_PROJECTION)
        glPushMatrix()
        glLoadIdentity()
        # Because of this, we need to draw in reverse order
        gluOrtho2D(0, size.width, 0, size.height)
        
        glMatrixMode(GL_MODELVIEW)
        glPushMatrix()
        glLoadIdentity()
        
        seqOrder = None
        if self.__drawOnlyIndex == -1:
            seqOrder = self.seqOrder
        else:
            seqOrder = [self.__drawOnlyIndex]
    
        # position offset: color    
        colors = {
                  (1, 0): Color(Color.RGB, 0.20, 0.20, 0.20),
                  (0, 1): Color(Color.RGB, 0.20, 0.20, 0.20),
                  (-1, 0): Color(Color.RGB, 0.20, 0.20, 0.20),
                  (0, -1): Color(Color.RGB, 0.20, 0.20, 0.20),
                  (1, 1): Color(Color.RGB, 0.20, 0.20, 0.20),
                  (-1, -1): Color(Color.RGB, 0.20, 0.20, 0.20),
                  (1, -1): Color(Color.RGB, 0.20, 0.20, 0.20),
                  (-1, 1): Color(Color.RGB, 0.20, 0.20, 0.20)
                 }        
        
        # Draw in reverse order because that's the way the world is setup in OpenGL
        revSeqOrder = list(seqOrder)
        revSeqOrder.reverse()
        for i in revSeqOrder:
            seq = self.__aln.getSeqStringByIndex(i)
            
            glPushMatrix()
            blockWidth = float(size.width) / length
            blockHeight = float(size.height) / self.__numSeqs
            
            blockDifference = 0
            
            glScalef(blockWidth / 104.76, blockHeight / 104.76 / 2, 0)
            glTranslatef(0, 104.76 / 2, 0)
            
            for i in range(self.start, self.end):
                #for position, color in colors.iteritems():
                    #print "Drawing at %d, %d with color %s" % (position[0], position[1], str(color))
                    #self.strokeCharacterInBlock(seq[i], position, color, blockWidth, blockHeight)
                
                glLineWidth(2.7)
                self.strokeCharacterInBlock(seq[i], (0, 1), Color(Color.RGB, 0.04, 0.04, 0.04), blockWidth, blockHeight)
                self.strokeCharacterInBlock(seq[i], (0, -1), Color(Color.RGB, 0.04, 0.04, 0.04), blockWidth, blockHeight)
                
                glLineWidth(0.6)
                # Stroke the real character in white
                self.strokeCharacterInBlock(seq[i], (0, 0), Color(Color.RGB, 0.5, 1, 1), blockWidth, blockHeight)
                    
                glTranslatef(104.76, 0, 0)

            glPopMatrix()
            glTranslatef(0, (float(size.height) / self.__numSeqs), 0)

        # Put it back as we found it
        glMatrixMode(GL_PROJECTION)
        glPopMatrix()
        glMatrixMode(GL_MODELVIEW)
        glPopMatrix()
        
        glDisable(GL_BLEND);
        glDisable(GL_LINE_SMOOTH);
        
    def strokeCharacterInBlock(self, char, position, color, blockWidth, blockHeight):
        glPushMatrix()
        
        # Move position *PIXELS*
        glTranslatef(position[0] * (104.76 / blockWidth), position[1] * (104.76 / blockHeight), 0)
        
        if (blockWidth / blockHeight) > 1:
            # Center the letter in the cell if it's being rendered at less
            # than the whole block
            glTranslatef((104.76 / 2) - ((107.76 * (blockHeight / blockWidth)) / 2.0), 0, 0)
            glScalef(blockHeight / blockWidth, 1, 1)
        
        color.provideToFunction(glColor3f)
        
        glutStrokeCharacter(GLUT_STROKE_MONO_ROMAN, ord(char))
        
        #glTranslatef(104.76, 0, 0)
        #glTranslatef(-104.76, 0, 0)
        glPopMatrix()
        
    def SetMinSize(self, tuple):
        preferences = getPreferences()
        
        maxMinWidth = preferences.getMaximumMinimumWidth()
        if tuple[0] > maxMinWidth:
            tuple = (maxMinWidth, tuple[1])
            
        InteractiveZoomableGLPanel.SetMinSize(self, tuple)