import wx

from display.openglhelper import *
from display.color import *
from display.colorkey import *

from feature.drawablefeature import *

from feature.cdd import *

from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *

class TraditionalTrackDrawer(GLPanel, Zoomable):
    def __init__(self, parent, id, pos = wx.DefaultPosition, size = wx.DefaultSize, zoom = None, seqRecord = None):
        self.__init = False
        GLPanel.__init__(self, parent, id, pos, size)
        Zoomable.__init__(self, zoom)
    
        self.__seqRecord = seqRecord
        
        self.__featureTracks = {}
        self.__determineFeatureTracks()
    
        self.SetMinSize((len(self.__seqRecord.seq), 18 * self.__numTracks))
    
        self.Bind(wx.EVT_SIZE, self.OnSize)
        self.Bind(wx.EVT_PAINT, self.OnPaint)
        
        self.__init = True
       
    def __determineFeatureTracks(self):
        tracks = {}
        
        for feat in self.__seqRecord.features:
            print feat
            # TODO: Implement type of feature checking here too (LOW PRIORITY)
            if not feat.isTrackFeature():
                continue
            
            seqLength = len(self.__seqRecord.seq)
            
            index = 0
            tryTrack = 0
            while index < seqLength:
                if feat.determineAtLocation(index) & DrawableFeature.LOCATION_AT:
                    if tryTrack not in tracks:
                        tracks[tryTrack] = {}
                            
                    if index in tracks[tryTrack]:
                        # Conflict, try again!
                        tryTrack += 1
                        index = 0
                        continue
                    
                index += 1
                
            for i in range(seqLength):
                if feat.determineAtLocation(i) & DrawableFeature.LOCATION_AT:
                    tracks[tryTrack][i] = 1
            
            self.__featureTracks[feat.id] = tryTrack
        
        self.__numTracks = len(tracks)
       
    def setZoom(self, start, end, fromApplication):
        if self.__init and not Zoomable.zoomChanged(self, start, end):
            return
        
        Zoomable.setZoom(self, start, end, fromApplication)
        
        self.Refresh()
        
    def OnSize(self, event):
        self.Refresh()
        
        event.Skip()
        
    def OnPaint(self, event):
        self.SetCurrent()
        dc = wx.PaintDC(self)
        
        self.setupViewport()

        size = self.GetClientSize()
        self.scales = (size.width, size.height)
        self.setUserScale(self.scales[0], self.scales[1])
     
        self.clearModelViewMatrix()
        
        # Background to grey-ish
        self.clearColor(0.75, 0.75, 0.75, 0)
        
        self.drawFeatures()
        
        self.flush()
        
        self.SwapBuffers()
        
    def drawFeatures(self):
        colorKey = ColorKey(None, -1)
        
        features = self.__seqRecord.features
        
        for feat in features:
            # Ignore non-track features
            if not feat.isTrackFeature():
                continue
            
            # TODO: Get colors in a sane way (HIGH PRIORITY)
            color = None
            if 'color' in feat.qualifiers:
                color = colorKey.getByOffset(feat.qualifiers['color'])
            else:
                color = colorKey.get(feat.id)
                
            color = color[int(len(color) / 2)] # Moderate brightness
            
            if len(feat.sub_features) > 0:
                for feat2 in feat.sub_features:
                    loc = feat2.location
                    self.drawTrack(loc.start.position, loc.end.position, self.__featureTracks[feat.id], feat.qualifiers['name'], color)
            else:
                loc = feat.location
                self.drawTrack(loc.start.position, loc.end.position, self.__featureTracks[feat.id], feat.qualifiers['name'], color)
            
    def drawTrack(self, start, end, track, label, color = None):
        size = self.GetClientSize()
        blockWidth = float(size.width) / (self.end - self.start)
        trackHeight = float(size.height) / self.__numTracks

        colorBlack = Color(Color.RGB, 0, 0, 0)
        colorWhite = Color(Color.RGB, 1, 1, 1)

        if color:
            color.provideToFunction(glColor3f)
        else:
            colorBlack.provideToFunction(glColor3f)
        
        startPosition = blockWidth * (start - self.start)
        endPosition = blockWidth * (end - self.start)        
        
        ## SQUARE
        
        glRectf(startPosition, track * trackHeight, endPosition, (track + 1) * trackHeight)
        
        ## TEXT
        
        colorBlack.provideToFunction(glColor3f)
        
        glLineWidth(2.0)
        
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        glEnable(GL_BLEND);
        glEnable(GL_LINE_SMOOTH);
        
        glMatrixMode(GL_PROJECTION)
        glPushMatrix()
        glLoadIdentity()
        gluOrtho2D(0, size.width, 0, size.height)        
        
        glMatrixMode(GL_MODELVIEW)
        glPushMatrix()        
        glLoadIdentity()
        
        leftOverSpace = (endPosition - startPosition)
        leftOverSpace -= (1 * 2)
        
        letterWidth = min(10, leftOverSpace / len(label))

        # If it's <= 2 letters, just try to draw all of them no matter what
        if len(label) > 2:
            trunced = False
            while letterWidth < 4:
                trunced = True
                
                label = label[0:-1]
                if len(label) > 0:
                    letterWidth = leftOverSpace / len(label)
                else:
                    label = '.'
                    letterWidth = leftOverSpace
                    break
            
            #if trunced:
            #    label += '...'
            
        letterSpace = letterWidth * len(label)     
        
        startTextPosition = startPosition
        startTextPadding = (((endPosition - startPosition) / 2) - (letterSpace / 2))
        startTextPosition += startTextPadding

        glTranslatef(startTextPosition, trackHeight * (self.__numTracks - track - 1) + (trackHeight / 3), 0)
        glScalef(letterSpace / len(label) / 104.76, trackHeight / 104.76 / 2, 0)

        for char in label:
            glutStrokeCharacter(GLUT_STROKE_MONO_ROMAN, ord(char))
        
        ## BACK TO NORMAL
        
        glMatrixMode(GL_MODELVIEW)
        glPopMatrix()
        glMatrixMode(GL_PROJECTION)
        glPopMatrix()
        
        glLineWidth(1.0)