import wx
import copy

from display.applicationhelper import *

class WxHelper:
    def drawRectangle(self, dc, rect):
        dc.DrawRectangleRect(rect)
        
    def setSolidPen(self, dc, color, width = None):
        solidPen = color.provideToFunctionAsWxColour(wx.Pen)
        
        if width is not None:
            solidPen.SetWidth(width)
            
        dc.SetPen(solidPen)
        
    def setTransparentPen(self, dc):
        dc.SetPen(wx.TRANSPARENT_PEN)
        
    def setSolidBrush(self, dc, color):
        solidBrush = color.provideToFunctionAsWxColour(wx.Brush)
        dc.SetBrush(solidBrush)
        
    def setTransparentBrush(self, dc):                
        dc.SetBrush(wx.TRANSPARENT_BRUSH)
        
    def drawFilledRectangle(self, dc, rect, color):
        self.setSolidBrush(dc, color)
        self.setTransparentPen(dc)
        
        self.drawRectangle(dc, rect)
        
    def drawUnfilledRectangle(self, dc, rect, color):
        self.setTransparentBrush(dc)
        self.setSolidPen(dc, color)
        
        self.drawRectangle(dc, rect)
        
    def drawFilledCircle(self, dc, centerPoint, radius, color):
        self.setSolidBrush(dc, color)
        self.setTransparentPen(dc)
        
        self.drawCircle(dc, centerPoint, radius)
        
    def drawUnfilledCircle(self, dc, centerPoint, radius, color):
        self.setTransparentBrush(dc)
        self.setSolidPen(dc, color)
        
        self.drawCircle(dc, centerPoint, radius)        
        
    def drawCircle(self, dc, centerPoint, radius):
        center = rect.getCenter()
        
        dc.DrawCirclePoint(centerPoint, radius)
        
    def drawFilledRoundedRectangle(self, dc, rect, radius, color):
        self.setSolidBrush(dc, color)
        self.setTransparentPen(dc)
        
        self.drawRoundedRectangle(dc, rect, radius)
        
    def drawUnfilledRoundedRectangle(self, dc, rect, radius, color):
        self.setTransparentBrush(dc)
        self.setSolidPen(dc, color)
        
        self.drawRoundedRectangle(dc, rect, radius)        
        
    def drawRoundedRectangle(self, dc, rect, radius):
        dc.DrawRoundedRectangleRect(rect, radius)

    def drawLine(self, dc, pt1, pt2, color, width = None):
        self.setSolidPen(dc, color, width)
        
        dc.DrawLinePoint(pt1, pt2)

    def drawFilledPolygon(self, dc, points, color):
        self.setSolidBrush(dc, color)
        self.setTransparentPen(dc)
        
        self.drawPolygon(dc, points)
        
    def drawPolygon(self, dc, points):
        dc.DrawPolygon(points)
        
    def drawUnfilledPolygon(self, dc, points, color):
        self.setSolidPen(dc, color)
        self.setTransparentBrush(dc)
        
        self.drawPolygon(dc, points)

    def drawFilledRectangleList(self, dc, list, brushes = None):
        self.setTransparentPen(dc)
        
        self.drawRectangleList(dc, list, None, brushes)
        
    def drawUnfilledRectangleList(self, dc, list, pens = None):
        self.setTransparentBrush(dc)
        
        self.drawRectangleList(dc, list, pens, None)        
        
    def drawRectangleList(self, dc, list, pens = None, brushes = None):
        dc.DrawRectangleList(list, pens, brushes)

    def pointInRect(self, pt, rect):
        if pt.x >= rect.x and pt.y >= rect.y and pt.x <= (rect.x + rect.width) and pt.y <= (rect.y + rect.height):
            return True
        else:
            return False
       
    def setTextForeground(self, dc, color):
        color.provideToFunctionAsWxColour(dc.SetTextForeground)
        
    def drawTextColor(self, dc, pt, font, text, color):
        self.setTextForeground(dc, color)
        
        self.drawText(dc, pt, font, text)
        
    def drawText(self, dc, pt, font, text):
        dc.SetFont(font)
        dc.DrawTextPoint(text, pt)
        
    def getEventAbsolutePosition(self, event):
        eventObjPosition = event.GetEventObject().GetPosition()
        eventPosition = event.GetPosition()
        
        return wx.Point(eventObjPosition.x + eventPosition.x, eventObjPosition.y + eventPosition.y)
    
    def pointInRectangle(self, pt, rect):
        return (
                pt.x >= rect.x and
                pt.y >= rect.y and
                pt.x <= rect.x + rect.width and
                pt.y <= rect.y + rect.height
                )
    
    def limitCoordsToSize(self, point, size):
        newPoint = copy.copy(point)
        if newPoint.x < 0:
            newPoint.x = 0
        elif newPoint.x > size.width:
            newPoint.x = size.width
            
        if newPoint.y < 0:
            newPoint.y = 0
        elif newPoint.y > size.height:
            newPoint.y = size.height
            
        return newPoint

    def getTextExtent(self, dc, font, text):
        dc.SetFont(font)
        return dc.GetTextExtent(text)

    def centerTextPoint(self, dc, rect, font, text):
        appHelper = getApplicationHelper()
        width, height = self.getTextExtent(dc, font, text)
        scaleX, scaleX = dc.GetUserScale()
        
        widthInU = width / scaleX
        heightInU = height / scaleX
        
        pt = wx.Point(appHelper.floor(rect.x + (rect.width / 2.0 - widthInU / 2.0)), int(rect.y + (rect.height / 2.0 - heightInU / 2.0)))
           
        return pt
    
    def displayErrorMessage(self, parent, message, caption = ""):
        dialog = wx.MessageDialog(parent, message, caption, wx.OK | wx.CENTRE | wx.ICON_ERROR)
        return dialog.ShowModal()

    def displayInfoMessage(self, parent, message, caption = ""):
        dialog = wx.MessageDialog(parent, message, caption, wx.OK | wx.CENTRE | wx.ICON_INFORMATION)
        return dialog.ShowModal()
        
    def displayYesNoMessage(self, parent, message, caption = ""):
        dialog = wx.MessageDialog(parent, message, caption, wx.YES_NO | wx.YES_DEFAULT | wx.ICON_QUESTION)
        return dialog.ShowModal()
        
    def setSizeBasedOnMax(self, windows, orientation = wx.HORIZONTAL):        
        max = 0
        
        for win in windows:
            size = win.GetSize()
            
            compare = None
            if orientation == wx.HORIZONTAL:
                compare = size.width
            else:
                compare = size.height
                
            if compare > max:
                max = compare
                
        for win in windows:
            size = win.GetSize()
            
            newSize = None
            if orientation == wx.HORIZONTAL:
                newSize = wx.Size(max, size.height)
            else:
                newSize = wx.Size(size.width, max)
                
            win.SetSize(newSize)
            
    def getAlnFilesFromDialog(self, parent, message, defaultDirectory, defaultFile, wildcard, style):
        appHelper = getApplicationHelper()
        
        if defaultDirectory is None:
            defaultDirectory = appHelper.getDefaultAlignmentDirectory()
            
        if defaultFile is None:
            defaultFile = ""
            
        if wildcard is None:
            wildcard = "ClustalW Alignment Files (*.aln)|*.aln|PHYLIP Alignment Files (*.phylip)|*.phylip"
        
        dlg = wx.FileDialog(parent, message, defaultDirectory, defaultFile, wildcard, style)
    
        re = None
        if dlg.ShowModal() == wx.ID_OK:
            re = dlg.GetPaths()
        else:
            re = []
            
        dlg.Destroy()
        return re       
        
    def refreshAll(self, windows):
        for win in windows:
            win.Refresh()
    
    def makeBoldStaticText(self, parent, id, label, **args):
        ctrl = wx.StaticText(parent, id, label, **args)
        font = ctrl.GetFont()
        font.SetWeight(wx.FONTWEIGHT_BOLD)
        ctrl.SetFont(font)
        
        return ctrl
    
helper = None
def getWxHelper():
    global helper
    
    if helper is None:
        helper = WxHelper()
        
    return helper

class ContextMenuableWxPanel(wx.Panel):
    def __init__(self, parent, id, pos = wx.DefaultPosition, size = wx.DefaultSize, style = 0):
        wx.Panel.__init__(self, parent, id, pos, size, style)
        
        self.__menu = None
        
        self.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDown)
    
    def setContextMenu(self, menu):
        self.__menu = menu
        
    def OnRightDown(self, event):
        if self.__menu is None:
            return
        
        self.PopupMenu(self.__menu, event.GetPosition())