from Bio.SeqFeature import FeatureLocation

import urllib2
import re

from feature.featurefetcher import *
from feature.featureutilities import *
from feature.bindn import *

class bindnFetcher(FeatureFetcher):
    def __init__(self):
        FeatureFetcher.__init__(self)
        
        self.setRelatedFeature(bindn)
        self.setSupportedSequences(FeatureFetcher.PROTEIN_SUPPORT)
    
    def getName(self):
        return "Predicted DNA binding sites (bindn)"
    
    def fetchesTrackFeature(self):
        return False    
    
    def fetch(self, seq):
        featUtils = getFeatureUtilities()
        
        reqURL = "http://129.130.115.71:9999/cgi-bin/bindn.pl?"
        
        reqURL += "seq=" + seq.replace("-", "")
        reqURL += "&qtype=dna"
        reqURL += "&vtype=sp"
        reqURL += "&val=80"
        reqURL += "&submit=Submit+Query"
        
        req = urllib2.Request(reqURL)
        res = urllib2.urlopen(req)
        
        bindns = re.findall("^(\\d+)\\s+([A-Z])\\s+([0-9.-]+)\\s+([+-])\\s+([0-9.-]+)$", res.read(), re.MULTILINE)
    
        features = []
        
        for site in bindns:
            if (site[3] == "+"):
                # -1 to get zero-based locations
                loc = FeatureLocation(int(site[0]) - 1, int(site[0]) - 1)
                # Even 0-length things need to adjust for gaps before them
                loc = featUtils.adjustForGaps(loc, seq)

                feat = self.makeFeature([loc], 'bindn', '', None, '', {'confidence': site[4]})
                features.append(feat)
        
        return features