from Bio.SeqFeature import FeatureLocation

import urllib2
import re

from feature.featureutilities import *
from feature.featurefetcher import *

from feature.cdd import *

class CDDFetcher(FeatureFetcher):
    def __init__(self):
        FeatureFetcher.__init__(self)
        
        self.setRelatedFeature(CDD)
        self.setSupportedSequences(FeatureFetcher.PROTEIN_SUPPORT)
        
    def getName(self):
        return "NCBI Conserved Domain Database"
    
    def fetchesTrackFeature(self):
        return True
    
    def fetch(self, seq):
        featUtils = getFeatureUtilities()
        
        reqURL = "http://www.ncbi.nlm.nih.gov/Structure/cdd/wrpsb.cgi?db=cdd&seqinput=" + seq.replace("-", "") + "&FULL"
        
        req = urllib2.Request(reqURL)
        res = urllib2.urlopen(req)
        
        results = res.read()
        
        cds = re.findall("value=\"(\\d+),(\\d+),(\\d+),([0-9\\.\\-e]+)\"", results)
        
        hash = {}
        for cd in cds:
            if cd[0] not in hash:
                hash[cd[0]] = []
            
            if cd not in hash[cd[0]]:
                hash[cd[0]].append(cd)
        
        # Pull out names
        # cddescs = re.findall("<td>(\\d+)</td>\\s+<td>([A-Za-z0-9_|-]+)</td>\\s+<td>([A-Za-z0-9_|-]+)</td>\\s+<td>([0-9.e-]+)</td>\\s+<td>([0-9.e-]+)</td>\\s+<td>([0-9.e-]+)</td>\\s+<td>([0-9.e-]+)</td>\\s+<td>([0-9e.-]+)</td>\\s+<td>([A-Za-z0-9_|-]+)</td>\\s+<td>([A-Za-z0-9_|-]+)</td>\\s+<td>([0-9e.-]+)</td>\\s+<td>([0-9,]+)</td>\\s+<td>(.+?)</td>", results)
	cddescs = re.findall("<tr><td>(\\d+)</td>\\s+<td>([A-Za-z0-9_|-]+)</td>\\s+<td>([A-Za-z0-9_|-]+)</td>\\s+<td>(.+?)</td>\\s+<td>(.+?)</td>\\s+<td>(true|false)</td>\\s+<td>(true|false)</td>\\s+<td>(true|false)</td>\\s+<td>([0-9.e-]+)</td>\\s+<td>([0-9.e-]+)</td>\\s+<td>([0-9.e-]+)</td>\\s+<td>([0-9.e-]+)</td>", results);
        qualHash = {}
        evalueSum = {}
        evalueNum = {}
        for desc in cddescs:
            # 0 = ID
            # 1 = DB ID
            # 2 = name
            # 3 = description
            # 9 = E-Value

            # (name, dbID, description)
            qualHash[desc[0]] = (desc[2], desc[1], desc[3])
            
            if desc[0] in evalueSum:
                evalueSum[desc[0]] += float(desc[9])
                evalueNum[desc[0]] += 1
            else:
                evalueSum[desc[0]] = float(desc[9])
                evalueNum[desc[0]] = 1
            
        features = []
        
        for values in hash.itervalues():
            locs = []
            for cd in values:
                # -1 to get zero-based locations
                loc = FeatureLocation(int(cd[1]) - 1, int(cd[2]) - 1)
                loc = featUtils.adjustForGaps(loc, seq)
                
                locs.append(loc)
            
            qualifiers = {}
            if cd[0] in qualHash:    
                qualTuple = qualHash[cd[0]]
                
                qualifiers['name'] = qualTuple[0]
                qualifiers['dbID'] = qualTuple[1]
                qualifiers['description'] = qualTuple[2]
                
                if cd[0] in evalueSum:
                    qualifiers['e-value'] = evalueSum[cd[0]] / evalueNum[cd[0]]
                    
                qualifiers['url'] = 'http://www.ncbi.nlm.nih.gov/Structure/cdd/cddsrv.cgi?uid=' + qualTuple[1]
                
            feat = self.makeFeature(locs, 'CDS', '', None, cd[0], qualifiers)
            features.append(feat)
            
        return features
