from Bio.SeqFeature import SeqFeature, FeatureLocation

class DrawableFeature(SeqFeature):
    LOCATION_NO = 1
    LOCATION_START = 2
    LOCATION_END = 4
    LOCATION_AT = 8
    
    def __init__(self, *args):
        SeqFeature.__init__(self, *args)
        
        self.__locationDict = {}
        
        self.firstStartLoc = -1
    
    def __buildLocationDictsForLocation(self, location):
        start = location.start.position
        end = location.end.position
        
        self.__addLocationMarker(start, self.LOCATION_START)
        if self.firstStartLoc == -1 or start < self.firstStartLoc:
            self.firstStartLoc = start
        
        self.__addLocationMarker(end, self.LOCATION_END)
        
        for i in range(start, end + 1):
            self.__addLocationMarker(i, self.LOCATION_AT)
    
    def __addLocationMarker(self, index, marker):
        if index not in self.__locationDict:
            self.__locationDict[index] = 0
            
        self.__locationDict[index] |= marker
    
    def __buildLocationDicts(self):
        self.__startLocations = {}
        self.__endLocations = {}
        self.__atLocations = {}
        
        if len(self.sub_features) > 0:
            for feat in self.sub_features:
                loc = feat.location
                self.__buildLocationDictsForLocation(loc)    
        elif self.location is not None:
            loc = self.location
            self.__buildLocationDictsForLocation(loc)
            
    def getAtLocationDict(self):
        return self.__atLocations
    
    def determineAtLocation(self, index):
        if len(self.__locationDict) == 0:
            self.__buildLocationDicts()
            
        if index in self.__locationDict:
            return self.__locationDict[index]
        else:
            return self.LOCATION_NO
        
    def getFirstStartLocation(self):
        if len(self.__locationDict) == 0:
            self.__buildLocationDicts()
        
        return self.firstStartLoc
    
    def isTrackFeature(self):
        return False
    
    def __cmp__(self, other):
        return cmp(self.getFirstStartLocation(), other.getFirstStartLocation())