from feature.featureutilities import *

from Bio.SeqFeature import FeatureLocation

class FeatureFetcher:
    __relatedFeature = None
    
    DNA_SUPPORT = 1
    PROTEIN_SUPPORT = 2
    
    def __init__(self):
        self.__sequenceSupport = 0
        
        pass
    
    def setSupportedSequences(self, support):
        self.__sequenceSupport = support
    
    def getSupportedSequences(self):
        return self.__sequenceSupport
    
    def getName(self):
        raise NotImplementedError("Should be overridden in child class")
    
    def setRelatedFeature(self, featType):
        self.__relatedFeature = featType
        
    def getRelatedFeature(self):
        return self.__relatedFeature
    
    def fetch(self, seq):
        raise NotImplementedError("Should be overridden in child class")
    
    def makeFeature(self, locations = [], type = '', location_operator = '', strand = None, id = '<unknown id>', qualifiers = {}, ref = None, ref_db = None):       
        # id = int(id)
        
        featType = self.getRelatedFeature()
        if featType is None:
            raise TypeError("Related feature not set")
        
        if len(locations) == 1:
            instance = featType(locations[0], type, location_operator, strand, id, qualifiers, [], ref, ref_db)
            instance.qualifiers = qualifiers
        else:
            subfeats = []
            
            for loc in locations:
                subfeats.append(featType(loc, type, location_operator, strand, id, qualifiers, [], ref, ref_db))
                
            instance = featType(None, type, location_operator, strand, id, qualifiers, subfeats, ref, ref_db)
            instance.sub_features = subfeats
            instance.qualifiers = qualifiers
            
        return instance