from feature.cddfetcher import *
from feature.bindnfetcher import *

class Preferences:
    SCORE_METHOD_BLOSUM62 = 1
    SCORE_METHOD_IDENTITY = 2
    
    SEQUENCE_TYPE_PROTEIN = 1
    SEQUENCE_TYPE_DNA = 2
    
    def __init__(self):
        # TODO: Save these somehow? (LOW PRIORITY)
        
        self.__fetchFeatures = True
        
        self.setScoreMethod(Preferences.SCORE_METHOD_BLOSUM62)
        self.setSequenceType(Preferences.SEQUENCE_TYPE_PROTEIN)
        
        allFetchers = [CDDFetcher(), bindnFetcher()]
        self.__allFetchers = {}
        self.__enabledFetchers = []
        for fetcher in allFetchers:
            self.__allFetchers[fetcher.getName()] = fetcher
            self.__enabledFetchers.append(fetcher.getName())
        
        # bindn isn't very important
        self.toggleFetcher('Predicted DNA binding sites (bindn)')
        
    def getFetchFeatures(self):    
        return self.__fetchFeatures
    
    def setFetchFeatures(self, fetch):
        self.__fetchFeatures = fetch
    
    def getEnabledFetchers(self):
        re = []
        for fetName in self.__allFetchers:
            if self.fetcherEnabled(fetName):
                re.append(self.__allFetchers[fetName])
                
        return re
    
    def getAllFetchers(self):
        return self.__allFetchers.values()
        
    def getFetcher(self, name):
        return self.__allFetchers[name]
        
    def fetcherEnabled(self, name):
        return (name in self.__enabledFetchers)
        
    def toggleFetcher(self, name):
        if self.fetcherEnabled(name):
            self.__enabledFetchers.remove(name)
        else:
            self.__enabledFetchers.append(name)   
            
    def getScoreMethod(self): 
        return self.__scoreMethod
    
    def setScoreMethod(self, method):
        self.__scoreMethod = method
        
    def setSequenceType(self, type):
        self.__sequenceType = type
        
    def getSequenceType(self):
        return self.__sequenceType
    
    def getMinimumDrawableCharacterWidth(self):
        return 10.0
    
    def getInitialNumberDisplayedFeatures(self):
        return 4
    
    def getMaximumMinimumWidth(self):
        return 750
    
    def getMinimumTrackHeight(self):
        return 12
    
    def getFetcherRetries(self):
        return 3
    
    def getFrameInitialSize(self):
        return (797, 597)
    
    def getNumberOfNotifyFlashes(self):
        return 5
    
    def getNotifyTimerFlashDelay(self):
        return 300
    
    def getFloatRangeSizerMaxValue(self):
        return 200
    
preferences = None
def getPreferences():
    global preferences
    if preferences is None:
        preferences = Preferences()
        
    return preferences